
from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .msh_control_info import MshControlInfo


@JsonMap({"msh_control_info": "MSHControlInfo"})
class Hl7ControlInfo(BaseModel):
    """Hl7ControlInfo

    :param msh_control_info: msh_control_info, defaults to None
    :type msh_control_info: MshControlInfo, optional
    """

    def __init__(self, msh_control_info: MshControlInfo = SENTINEL, **kwargs):
        """Hl7ControlInfo

        :param msh_control_info: msh_control_info, defaults to None
        :type msh_control_info: MshControlInfo, optional
        """
        if msh_control_info is not SENTINEL:
            self.msh_control_info = self._define_object(msh_control_info, MshControlInfo)
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..execution_record import ExecutionRecordService
from ...models import ExecutionRecord, ExecutionRecordQueryResponse, ExecutionRecordQueryConfig


class ExecutionRecordServiceAsync(ExecutionRecordService):
    """
    Async Wrapper for ExecutionRecordServiceAsync
    """

    def query_execution_record(
        self, request_body: ExecutionRecordQueryConfig = None
    ) -> Awaitable[Union[ExecutionRecordQueryResponse, str]]:
        return to_async(super().query_execution_record)(request_body)

    def query_more_execution_record(
        self, request_body: str
    ) -> Awaitable[Union[ExecutionRecordQueryResponse, str]]:
        return to_async(super().query_more_execution_record)(request_body)

    def async_get_execution_record(
        self, id_: str
    ) -> Awaitable[Union[ExecutionRecord, str]]:
        return to_async(super().async_get_execution_record)(id_)

    def get_execution_record(
        self, id_: str
    ) -> Awaitable[Union[ExecutionRecord, str]]:
        return to_async(super().get_execution_record)(id_)




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({"generic_connector_record_id": "genericConnectorRecordId"})
class Document(BaseModel):
    """Document

    :param generic_connector_record_id: generic_connector_record_id, defaults to None
    :type generic_connector_record_id: str, optional
    """

    def __init__(self, generic_connector_record_id: str = SENTINEL, **kwargs):
        """Document

        :param generic_connector_record_id: generic_connector_record_id, defaults to None
        :type generic_connector_record_id: str, optional
        """
        if generic_connector_record_id is not SENTINEL:
            self.generic_connector_record_id = generic_connector_record_id
        self._kwargs = kwargs




"""
Defines the SENTINEL object to be used to represent a nullable value.
"""
from typing import Any

SENTINEL = object()


def was_value_set(value: Any) -> bool:
    """Returns True if the value was set, False otherwise.

    :param value: The value to check.
    :type value: Any
    :return: True if the value was set, False otherwise.
    """
    return value is not SENTINEL




from __future__ import annotations
from typing import Optional
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .event_expression import EventExpression, EventExpressionGuard
from .event_simple_expression import EventSimpleExpression
from .event_grouping_expression import EventGroupingExpression


@JsonMap({})
class EventQueryConfigQueryFilter(BaseModel):
    """EventQueryConfigQueryFilter

    :param expression: expression
    :type expression: EventExpression
    """

    def __init__(self, expression: EventExpression, **kwargs):
        """EventQueryConfigQueryFilter

        :param expression: expression
        :type expression: EventExpression
        """
        self.expression = EventExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class EventQueryConfig(BaseModel):
    """EventQueryConfig

    :param query_filter: query_filter (optional)
    :type query_filter: Optional[EventQueryConfigQueryFilter]
    """

    def __init__(self, query_filter: Optional[EventQueryConfigQueryFilter] = None, **kwargs):
        """EventQueryConfig

        :param query_filter: query_filter (optional)
        :type query_filter: Optional[EventQueryConfigQueryFilter]
        """
        self.query_filter = self._define_object(
            query_filter, EventQueryConfigQueryFilter
        ) if query_filter is not None else None
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .gs_control_info import GsControlInfo
from .isa_control_info import IsaControlInfo


@JsonMap({"gs_control_info": "GSControlInfo", "isa_control_info": "isaControlInfo"})
class X12ControlInfo(BaseModel):
    """X12ControlInfo

    :param gs_control_info: gs_control_info, defaults to None
    :type gs_control_info: GsControlInfo, optional
    :param isa_control_info: isa_control_info, defaults to None
    :type isa_control_info: IsaControlInfo, optional
    """

    def __init__(
        self,
        gs_control_info: GsControlInfo = SENTINEL,
        isa_control_info: IsaControlInfo = SENTINEL,
        **kwargs,
    ):
        """X12ControlInfo

        :param gs_control_info: gs_control_info, defaults to None
        :type gs_control_info: GsControlInfo, optional
        :param isa_control_info: isa_control_info, defaults to None
        :type isa_control_info: IsaControlInfo, optional
        """
        if gs_control_info is not SENTINEL:
            self.gs_control_info = self._define_object(gs_control_info, GsControlInfo)
        if isa_control_info is not SENTINEL:
            self.isa_control_info = self._define_object(
                isa_control_info, IsaControlInfo
            )
        self._kwargs = kwargs




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import AtomWorkerLog, LogDownload


class AtomWorkerLogService(BaseService):

    @cast_models
    def create_atom_worker_log(
        self, request_body: AtomWorkerLog = None
    ) -> Union[LogDownload, str]:
        """Allows users to programmatically retrieve a link for downloading a given Runtime workers log.

        :param request_body: The request body., defaults to None
        :type request_body: AtomWorkerLog, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[LogDownload, str]
        """

        Validator(AtomWorkerLog).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/AtomWorkerLog",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return LogDownload._unmap(response)
        if content == "application/xml":
            return LogDownload._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    def download_atom_worker_log(
        self,
        request_body: AtomWorkerLog = None,
        max_retries: int = 10,
        initial_delay: float = 2.0,
    ) -> bytes:
        """Request and download Runtime worker logs.

        Combines the two-phase download process: submits the download request via
        create_atom_worker_log(), then polls the returned URL until the content is ready.

        :param request_body: The request body., defaults to None
        :type request_body: AtomWorkerLog, optional
        :param max_retries: Maximum number of polling attempts., defaults to 10
        :type max_retries: int
        :param initial_delay: Initial delay in seconds between retries., defaults to 2.0
        :type initial_delay: float
        :return: The raw log content as bytes.
        :rtype: bytes
        """
        result = self.create_atom_worker_log(request_body=request_body)
        if hasattr(result, "status_code") and str(result.status_code) == "504":
            raise ApiError("Runtime unavailable for log download", 504, result)
        if not hasattr(result, "url") or not result.url:
            raise ApiError("No download URL in response", 0, result)
        return self._poll_download_url(
            result.url, max_retries=max_retries, initial_delay=initial_delay
        )




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({"id_": "id"})
class ComponentInfo(BaseModel):
    """ComponentInfo

    :param id_: The ID of the component.The component ID is available in the Revision History dialog, which you can open from the Build page in the user interface. Potentially, this ID can return multiple components., defaults to None
    :type id_: str, optional
    :param version: Each saved configuration is assigned a Revision History number when you change a component on the Build tab. The component version is available in the Revision History dialog, which you can access from the Build page  service., defaults to None
    :type version: int, optional
    """

    def __init__(self, id_: str = SENTINEL, version: int = SENTINEL, **kwargs):
        """ComponentInfo

        :param id_: The ID of the component.The component ID is available in the Revision History dialog, which you can open from the Build page in the user interface. Potentially, this ID can return multiple components., defaults to None
        :type id_: str, optional
        :param version: Each saved configuration is assigned a Revision History number when you change a component on the Build tab. The component version is available in the Revision History dialog, which you can access from the Build page  service., defaults to None
        :type version: int, optional
        """
        if id_ is not SENTINEL:
            self.id_ = id_
        if version is not SENTINEL:
            self.version = version
        self._kwargs = kwargs



import copy
from typing import Union

from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import (
    CloudAttachmentProperties,
    CloudAttachmentPropertiesAsyncResponse,
    AsyncOperationTokenResult,
)


class CloudAttachmentPropertiesService(BaseService):

    @cast_models
    def update_cloud_attachment_properties(
        self, id_: str, request_body: CloudAttachmentProperties
    ) -> Union[CloudAttachmentProperties, str]:
        """Modifies one or more Cloud attachment properties.

        :param request_body: The request body. Must include container_name.
        :type request_body: CloudAttachmentProperties
        :param id_: id_
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[CloudAttachmentProperties, str]
        """

        Validator(CloudAttachmentProperties).validate(request_body)
        if not getattr(request_body, "container_name", None):
            raise ValueError("request_body.container_name is required")
        Validator(str).validate(id_)

        body = request_body
        if not getattr(request_body, "runtime_id", None):
            body = copy.copy(request_body)
            body.runtime_id = id_

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/CloudAttachmentProperties/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("POST")
            .set_body(body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return CloudAttachmentProperties._unmap(response)
        if content == "application/xml":
            return CloudAttachmentProperties._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def async_get_cloud_attachment_properties(
        self, id_: str
    ) -> Union[AsyncOperationTokenResult, str]:
        """Use the GET operation to return and view Cloud attachment properties.

        :param id_: id_
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AsyncOperationTokenResult, str]
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/async/CloudAttachmentProperties/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AsyncOperationTokenResult._unmap(response)
        if content == "application/xml":
            return AsyncOperationTokenResult._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def async_token_cloud_attachment_properties(
        self, token: str
    ) -> Union[CloudAttachmentPropertiesAsyncResponse, str]:
        """Send a second GET request with the token returned in the first GET request.

        :param token: Takes in the token from a previous call to return a result.
        :type token: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[CloudAttachmentPropertiesAsyncResponse, str]
        """

        Validator(str).validate(token)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/async/CloudAttachmentProperties/response/{{token}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("token", token)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return CloudAttachmentPropertiesAsyncResponse._unmap(response)
        if content == "application/xml":
            return CloudAttachmentPropertiesAsyncResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from enum import Enum
from .sentinel import was_value_set

# Mapping SDK class names to Boomi API @type names
# Needed for polymorphic types where SDK uses different casing than API
TYPE_NAME_MAPPING = {
    # MLLP Protocol
    'MllpCommunicationOptions': 'MLLPCommunicationOptions',
    'MllpSendSettings': 'MLLPSendSettings',
    'MllpsslOptions': 'MLLPSSLOptions',
    # HL7 Standard
    'Hl7PartnerInfo': 'HL7PartnerInfo',
    'Hl7Options': 'HL7Options',
    # FTP/SFTP Protocol
    'FtpCommunicationOptions': 'FTPCommunicationOptions',
    'FtpSendSettings': 'FTPSendSettings',
    'FtpsslOptions': 'FTPSSLOptions',
    'SftpCommunicationOptions': 'SFTPCommunicationOptions',
    'SftpSendSettings': 'SFTPSendSettings',
    'SftpsshOptions': 'SFTPSSHOptions',
    'SftpProxySettings': 'SFTPProxySettings',
    # HTTP Protocol
    'HttpCommunicationOptions': 'HTTPCommunicationOptions',
    'HttpSendSettings': 'HTTPSendSettings',
    'HttpsslOptions': 'HTTPSSLOptions',
    'HttpoAuthSettings': 'HTTPOAuthSettings',
    'HttpoAuth2Settings': 'HTTPOAuth2Settings',
    # AS2 Protocol
    'As2CommunicationOptions': 'AS2CommunicationOptions',
    'As2SendSettings': 'AS2SendSettings',
    'As2SendOptions': 'AS2SendOptions',
    'As2SslOptions': 'AS2SSLOptions',
    'As2PartnerInfo': 'AS2PartnerInfo',
    'As2MdnOptions': 'AS2MDNOptions',
    'As2MessageOptions': 'AS2MessageOptions',
    # OFTP Protocol
    'OftpCommunicationOptions': 'OFTPCommunicationOptions',
    'OftpSendSettings': 'OFTPSendSettings',
    'OftpPartnerInfo': 'OFTPPartnerInfo',
    # X12 Standard
    'X12PartnerInfo': 'X12PartnerInfo',
    'X12Options': 'X12Options',
    # EDIFACT Standard
    'EdifactPartnerInfo': 'EDIFACTPartnerInfo',
    'EdifactOptions': 'EDIFACTOptions',
}


class JsonMap:
    """
    A class decorator used to map adjusted attribute names to original JSON attribute names before a request,
    and vice versa after the request.

    Example:
    @JsonMapping({
        'adjusted_name': 'original_name',
        'adjusted_list': 'original_list'
    })
    class SomeClass(BaseModel):
        adjusted_name: str
        adjusted_list: List[OtherClass]

    :param mapping: A dictionary specifying the mapping between adjusted attribute names and original JSON attribute names.
    :type mapping: dict
    """

    def __init__(self, mapping):
        self.mapping = mapping

    def __call__(self, cls):
        """
        Transform the decorated class with attribute mapping capabilities.

        :param cls: The class to be decorated.
        :type cls: type
        :return: The decorated class.
        :rtype: type
        """
        cls.__json_mapping = self.mapping

        def _map(self):
            """
            Convert the object's attributes to a dictionary with mapped attribute names.

            :return: A dictionary with mapped attribute names and values.
            :rtype: dict
            """
            map = self.__json_mapping
            attribute_dict = vars(self)
            result_dict = {}

            # Add @type field for Boomi API polymorphic type support
            # This is required for nested objects in TradingPartnerComponent
            # Use mapping to convert SDK class names to API type names
            class_name = self.__class__.__name__
            result_dict['@type'] = TYPE_NAME_MAPPING.get(class_name, class_name)

            for key, value in attribute_dict.items():
                if key.startswith("_") or not was_value_set(value):
                    continue

                # Skip None values - Boomi API doesn't accept nulls
                if value is None:
                    continue

                if isinstance(value, list):
                    value = [v._map() if hasattr(v, "_map") else v for v in value]
                elif isinstance(value, Enum):
                    value = value.value
                elif hasattr(value, "_map"):
                    value = value._map()
                elif isinstance(value, bool):
                    # Keep as native bool (JSON serializer handles it)
                    pass
                elif isinstance(value, str) and value.lower() in ('true', 'false'):
                    # Convert string booleans to native bool
                    value = value.lower() == 'true'

                mapped_key = map.get(key, key)
                result_dict[mapped_key] = value

            return result_dict

        @classmethod
        def _unmap(cls, mapped_data):
            """
            Create an object instance from a dictionary with mapped attribute names.

            :param mapped_data: A dictionary with mapped attribute names and values.
            :type mapped_data: dict
            :return: An instance of the class with attribute values assigned from the dictionary.
            :rtype: cls
            """
            reversed_map = {v: k for k, v in cls.__json_mapping.items()}
            mapped_attributes = {}

            for key, value in mapped_data.items():
                # Handle Boomi typed-array format: ['BigInteger', 2575] -> 2575, ['Long', 3496] -> 3496
                # Only unwrap known Java type wrappers, not arbitrary [str, value] lists
                _JAVA_TYPE_NAMES = {
                    'BigInteger', 'BigDecimal', 'Long', 'Integer', 'Short', 'Byte',
                    'Float', 'Double', 'Boolean', 'String', 'Character',
                }
                if (isinstance(value, list) and len(value) == 2
                        and isinstance(value[0], str) and value[0] in _JAVA_TYPE_NAMES):
                    value = value[1]
                mapped_key = reversed_map.get(key, key)
                mapped_attributes[mapped_key] = value

            return cls(**mapped_attributes)

        # Only set _map if the class doesn't have its own custom implementation
        # This allows classes to override with their own serialization logic
        # Use __dict__ to check if the class itself (not parent) defines _map
        if '_map' not in cls.__dict__:
            cls._map = _map
        cls._unmap = _unmap

        return cls




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import (
    Connector,
    ConnectorBulkRequest,
    ConnectorBulkResponse,
    ConnectorQueryConfig,
    ConnectorQueryResponse,
)


class ConnectorService(BaseService):

    @cast_models
    def get_connector(self, connector_type: str) -> Union[Connector, str]:
        """You can only perform the GET operation by `type` and not by `name`.

         Send an HTTP GET where `accountId` is the ID of the authenticating account for the request and `connectorType` is the name of the connector subtype you are attempting to GET.

        For example, sending an HTTP GET to `https://api.boomi.com/api/rest/v1/accountId/Connector/database` returns `Database` type connectors available on the account.

        :param connector_type: The internal and unique identifier for connector type, such as `http`, `ftp`, `greatplains`. The [Component Metadata object](/api/platformapi#tag/ComponentMetadata) refers to this field as *subType*.
        :type connector_type: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[Connector, str]
        """

        Validator(str).validate(connector_type)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Connector/{{connectorType}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("connectorType", connector_type)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return Connector._unmap(response)
        if content == "application/xml":
            return Connector._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def bulk_connector(
        self, request_body: ConnectorBulkRequest = None
    ) -> Union[ConnectorBulkResponse, str]:
        """To learn more about `bulk`, refer to [Bulk GET operations](#section/Introduction/Bulk-GET-operations).

        :param request_body: The request body., defaults to None
        :type request_body: ConnectorBulkRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[ConnectorBulkResponse, str]
        """

        Validator(ConnectorBulkRequest).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Connector/bulk",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return ConnectorBulkResponse._unmap(response)
        if content == "application/xml":
            return ConnectorBulkResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_connector(
        self, request_body: ConnectorQueryConfig = None
    ) -> Union[ConnectorQueryResponse, str]:
        """For general information about the structure of QUERY filters, their sample payloads, and how to handle the paged results, refer to [Query filters](#section/Introduction/Query-filters) and [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body., defaults to None
        :type request_body: ConnectorQueryConfig, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[ConnectorQueryResponse, str]
        """

        Validator(ConnectorQueryConfig).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Connector/query",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return ConnectorQueryResponse._unmap(response)
        if content == "application/xml":
            return ConnectorQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_more_connector(
        self, request_body: str
    ) -> Union[ConnectorQueryResponse, str]:
        """To learn about using `queryMore`, refer to [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body.
        :type request_body: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[ConnectorQueryResponse, str]
        """

        Validator(str).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Connector/queryMore",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body, "text/plain")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return ConnectorQueryResponse._unmap(response)
        if content == "application/xml":
            return ConnectorQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..environment_extensions import EnvironmentExtensionsService
from ...models import (
    EnvironmentExtensions,
    EnvironmentExtensionsBulkResponse,
    EnvironmentExtensionsBulkRequest,
    EnvironmentExtensionsQueryResponse,
    EnvironmentExtensionsQueryConfig,
)


class EnvironmentExtensionsServiceAsync(EnvironmentExtensionsService):
    """
    Async Wrapper for EnvironmentExtensionsServiceAsync
    """

    def get_environment_extensions(
        self, id_: str
    ) -> Awaitable[Union[EnvironmentExtensions, str]]:
        return to_async(super().get_environment_extensions)(id_)

    def update_environment_extensions(
        self, id_: str, request_body: EnvironmentExtensions = None
    ) -> Awaitable[Union[EnvironmentExtensions, str]]:
        return to_async(super().update_environment_extensions)(id_, request_body)

    def bulk_environment_extensions(
        self, request_body: EnvironmentExtensionsBulkRequest = None
    ) -> Awaitable[Union[EnvironmentExtensionsBulkResponse, str]]:
        return to_async(super().bulk_environment_extensions)(request_body)

    def query_environment_extensions(
        self, request_body: EnvironmentExtensionsQueryConfig = None
    ) -> Awaitable[Union[EnvironmentExtensionsQueryResponse, str]]:
        return to_async(super().query_environment_extensions)(request_body)

    def query_more_environment_extensions(
        self, request_body: str
    ) -> Awaitable[Union[EnvironmentExtensionsQueryResponse, str]]:
        return to_async(super().query_more_environment_extensions)(request_body)




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .atom_security_policies_type import AtomSecurityPoliciesType


@JsonMap({"atom_id": "atomId"})
class AtomSecurityPolicies(BaseModel):
    """AtomSecurityPolicies

    :param atom_id: The ID of the Runtime cloud or Runtime cluster.
    :type atom_id: str
    :param browser: browser, defaults to None
    :type browser: AtomSecurityPoliciesType, optional
    :param common: common
    :type common: AtomSecurityPoliciesType
    :param runner: runner, defaults to None
    :type runner: AtomSecurityPoliciesType, optional
    :param worker: worker, defaults to None
    :type worker: AtomSecurityPoliciesType, optional
    """

    def __init__(
        self,
        atom_id: str,
        common: AtomSecurityPoliciesType,
        browser: AtomSecurityPoliciesType = SENTINEL,
        runner: AtomSecurityPoliciesType = SENTINEL,
        worker: AtomSecurityPoliciesType = SENTINEL,
        **kwargs,
    ):
        """AtomSecurityPolicies

        :param atom_id: The ID of the Runtime cloud or Runtime cluster.
        :type atom_id: str
        :param browser: browser, defaults to None
        :type browser: AtomSecurityPoliciesType, optional
        :param common: common
        :type common: AtomSecurityPoliciesType
        :param runner: runner, defaults to None
        :type runner: AtomSecurityPoliciesType, optional
        :param worker: worker, defaults to None
        :type worker: AtomSecurityPoliciesType, optional
        """
        self.atom_id = atom_id
        if browser is not SENTINEL:
            self.browser = self._define_object(browser, AtomSecurityPoliciesType)
        self.common = self._define_object(common, AtomSecurityPoliciesType)
        if runner is not SENTINEL:
            self.runner = self._define_object(runner, AtomSecurityPoliciesType)
        if worker is not SENTINEL:
            self.worker = self._define_object(worker, AtomSecurityPoliciesType)
        self._kwargs = kwargs




from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap(
    {
        "client_certificate": "clientCertificate",
        "component_filters": "componentFilters",
        "external_username": "externalUsername",
        "ip_filters": "ipFilters",
        "role_associations": "roleAssociations",
        "using_component_filters": "usingComponentFilters",
        "using_ip_filters": "usingIPFilters",
    }
)
class SharedWebServerUser(BaseModel):
    """SharedWebServerUser

    :param client_certificate: client_certificate, defaults to None
    :type client_certificate: str, optional
    :param component_filters: component_filters, defaults to None
    :type component_filters: List[str], optional
    :param external_username: external_username, defaults to None
    :type external_username: str, optional
    :param ip_filters: ip_filters, defaults to None
    :type ip_filters: List[str], optional
    :param role_associations: role_associations, defaults to None
    :type role_associations: List[str], optional
    :param token: token, defaults to None
    :type token: str, optional
    :param username: username
    :type username: str
    :param using_component_filters: using_component_filters, defaults to None
    :type using_component_filters: bool, optional
    :param using_ip_filters: using_ip_filters, defaults to None
    :type using_ip_filters: bool, optional
    """

    def __init__(
        self,
        username: str,
        client_certificate: str = SENTINEL,
        component_filters: List[str] = SENTINEL,
        external_username: str = SENTINEL,
        ip_filters: List[str] = SENTINEL,
        role_associations: List[str] = SENTINEL,
        token: str = SENTINEL,
        using_component_filters: bool = SENTINEL,
        using_ip_filters: bool = SENTINEL,
        **kwargs
    ):
        """SharedWebServerUser

        :param client_certificate: client_certificate, defaults to None
        :type client_certificate: str, optional
        :param component_filters: component_filters, defaults to None
        :type component_filters: List[str], optional
        :param external_username: external_username, defaults to None
        :type external_username: str, optional
        :param ip_filters: ip_filters, defaults to None
        :type ip_filters: List[str], optional
        :param role_associations: role_associations, defaults to None
        :type role_associations: List[str], optional
        :param token: token, defaults to None
        :type token: str, optional
        :param username: username
        :type username: str
        :param using_component_filters: using_component_filters, defaults to None
        :type using_component_filters: bool, optional
        :param using_ip_filters: using_ip_filters, defaults to None
        :type using_ip_filters: bool, optional
        """
        if client_certificate is not SENTINEL:
            self.client_certificate = client_certificate
        if component_filters is not SENTINEL:
            self.component_filters = component_filters
        if external_username is not SENTINEL:
            self.external_username = external_username
        if ip_filters is not SENTINEL:
            self.ip_filters = ip_filters
        if role_associations is not SENTINEL:
            self.role_associations = role_associations
        if token is not SENTINEL:
            self.token = token
        self.username = username
        if using_component_filters is not SENTINEL:
            self.using_component_filters = using_component_filters
        if using_ip_filters is not SENTINEL:
            self.using_ip_filters = using_ip_filters
        self._kwargs = kwargs




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import (
    SharedServerInformation,
    SharedServerInformationBulkRequest,
    SharedServerInformationBulkResponse,
)


class SharedServerInformationService(BaseService):

    @cast_models
    def get_shared_server_information(
        self, id_: str
    ) -> Union[SharedServerInformation, str]:
        """Retrieve Shared Server Information records for a specific single Runtime ID.

         You can retrieve Shared Server Information records only by an ordinary GET operation specifying a single Runtime ID or a bulk GET operation with a maximum of 100 Runtime IDs. This option is because the object ID for the Shared Server Information is not available currently (except by requesting the information from services). Therefore, this operation does not return the Shared Server Information object auth field.

        :param id_: The ID of the Runtime that is hosting the shared web server.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[SharedServerInformation, str]
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/SharedServerInformation/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return SharedServerInformation._unmap(response)
        if content == "application/xml":
            return SharedServerInformation._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def update_shared_server_information(
        self, id_: str, request_body: SharedServerInformation = None
    ) -> Union[SharedServerInformation, str]:
        """Updates a Shared Server Information object based on the supplied Runtime ID.

         - The UPDATE operation updates a Shared Server Information object based on the supplied Runtime ID. To clear a field, set the attribute corresponding to that field to an empty string.

         You must have the Runtime Management privilege to perform the UPDATE operation. If you have the Runtime Management Read Access privilege, you cannot update shared server information.

         It is not possible to set authToken through this operation. This operation generates a token if it requires authentication, but a token does not currently exist. The new authToken appears in the response.
         - If you specify sslCertificateId, the certificate must be accessible by the account making the request.
         -If you do not configure the Authentication Type and Ports, using the Shared Server Information object to update only the API Type of a Runtime fails. If you are the owner of a Runtime, Runtime cluster, or Runtime cloud, you must update the API Type, Authentication Type, and HTTP Port or HTTPS Port through the Shared Server Information object for the API to succeed. Runtime cloud attachments cannot update the HTTP Port or HTTPS Port.
         - If you configure the Authentication Type and Ports, you can use the Shared Server Information object to update only the API Type of a Runtime.
         - This API does not support the configuration of multiple authentication types on a Runtime.

        :param request_body: The request body., defaults to None
        :type request_body: SharedServerInformation, optional
        :param id_: The ID of the Runtime that is hosting the shared web server.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[SharedServerInformation, str]
        """

        Validator(SharedServerInformation).is_optional().validate(request_body)
        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/SharedServerInformation/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return SharedServerInformation._unmap(response)
        if content == "application/xml":
            return SharedServerInformation._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def bulk_shared_server_information(
        self, request_body: SharedServerInformationBulkRequest = None
    ) -> Union[SharedServerInformationBulkResponse, str]:
        """To learn more about `bulk`, refer to [Bulk GET operations](#section/Introduction/Bulk-GET-operations).

        :param request_body: The request body., defaults to None
        :type request_body: SharedServerInformationBulkRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[SharedServerInformationBulkResponse, str]
        """

        Validator(SharedServerInformationBulkRequest).is_optional().validate(
            request_body
        )

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/SharedServerInformation/bulk",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return SharedServerInformationBulkResponse._unmap(response)
        if content == "application/xml":
            return SharedServerInformationBulkResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .hl7_control_info import Hl7ControlInfo
from .hl7_options import Hl7Options


@JsonMap({"hl7_control_info": "HL7ControlInfo", "hl7_options": "HL7Options"})
class Hl7PartnerInfo(BaseModel):
    """Hl7PartnerInfo

    :param hl7_control_info: hl7_control_info, defaults to None
    :type hl7_control_info: Hl7ControlInfo, optional
    :param hl7_options: hl7_options, defaults to None
    :type hl7_options: Hl7Options, optional
    """

    def __init__(
        self,
        hl7_control_info: Hl7ControlInfo = SENTINEL,
        hl7_options: Hl7Options = SENTINEL,
        **kwargs,
    ):
        """Hl7PartnerInfo

        :param hl7_control_info: hl7_control_info, defaults to None
        :type hl7_control_info: Hl7ControlInfo, optional
        :param hl7_options: hl7_options, defaults to None
        :type hl7_options: Hl7Options, optional
        """
        if hl7_control_info is not SENTINEL:
            self.hl7_control_info = self._define_object(
                hl7_control_info, Hl7ControlInfo
            )
        if hl7_options is not SENTINEL:
            self.hl7_options = self._define_object(hl7_options, Hl7Options)
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..shared_web_server import SharedWebServerService
from ...models import (
    SharedWebServer,
    SharedWebServerBulkResponse,
    SharedWebServerBulkRequest,
)


class SharedWebServerServiceAsync(SharedWebServerService):
    """
    Async Wrapper for SharedWebServerServiceAsync
    """

    def get_shared_web_server(self, id_: str) -> Awaitable[Union[SharedWebServer, str]]:
        return to_async(super().get_shared_web_server)(id_)

    def update_shared_web_server(
        self, id_: str, request_body: SharedWebServer = None
    ) -> Awaitable[Union[SharedWebServer, str]]:
        return to_async(super().update_shared_web_server)(id_, request_body)

    def bulk_shared_web_server(
        self, request_body: SharedWebServerBulkRequest = None
    ) -> Awaitable[Union[SharedWebServerBulkResponse, str]]:
        return to_async(super().bulk_shared_web_server)(request_body)




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..atom import AtomService
from ...models import (
    Atom,
    AtomBulkResponse,
    AtomBulkRequest,
    AtomQueryResponse,
    AtomQueryConfig,
    AtomCountersAsyncResponse,
    AsyncOperationTokenResult,
    PersistedProcessPropertiesAsyncResponse,
)


class AtomServiceAsync(AtomService):
    """
    Async Wrapper for AtomServiceAsync
    """

    def create_atom(self, request_body: Atom = None) -> Awaitable[Union[Atom, str]]:
        return to_async(super().create_atom)(request_body)

    def get_atom(self, id_: str) -> Awaitable[Union[Atom, str]]:
        return to_async(super().get_atom)(id_)

    def update_atom(
        self, id_: str, request_body: Atom = None
    ) -> Awaitable[Union[Atom, str]]:
        return to_async(super().update_atom)(id_, request_body)

    def delete_atom(self, id_: str) -> Awaitable[None]:
        return to_async(super().delete_atom)(id_)

    def bulk_atom(
        self, request_body: AtomBulkRequest = None
    ) -> Awaitable[Union[AtomBulkResponse, str]]:
        return to_async(super().bulk_atom)(request_body)

    def query_atom(
        self, request_body: AtomQueryConfig = None
    ) -> Awaitable[Union[AtomQueryResponse, str]]:
        return to_async(super().query_atom)(request_body)

    def query_more_atom(
        self, request_body: str
    ) -> Awaitable[Union[AtomQueryResponse, str]]:
        return to_async(super().query_more_atom)(request_body)

    def async_token_atom_counters(
        self, token: str
    ) -> Awaitable[Union[AtomCountersAsyncResponse, str]]:
        return to_async(super().async_token_atom_counters)(token)

    def async_get_atom_counters(
        self, id_: str
    ) -> Awaitable[Union[AsyncOperationTokenResult, str]]:
        return to_async(super().async_get_atom_counters)(id_)

    def async_token_persisted_process_properties(
        self, token: str
    ) -> Awaitable[Union[PersistedProcessPropertiesAsyncResponse, str]]:
        return to_async(super().async_token_persisted_process_properties)(token)

    def async_get_persisted_process_properties(
        self, id_: str
    ) -> Awaitable[Union[AsyncOperationTokenResult, str]]:
        return to_async(super().async_get_persisted_process_properties)(id_)




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({"document_type": "documentType", "process_id": "processId"})
class ProcessingGroupPartnerDocumentRoute(BaseModel):
    """ProcessingGroupPartnerDocumentRoute

    :param document_type: document_type, defaults to None
    :type document_type: str, optional
    :param process_id: process_id, defaults to None
    :type process_id: str, optional
    """

    def __init__(
        self, document_type: str = SENTINEL, process_id: str = SENTINEL, **kwargs
    ):
        """ProcessingGroupPartnerDocumentRoute

        :param document_type: document_type, defaults to None
        :type document_type: str, optional
        :param process_id: process_id, defaults to None
        :type process_id: str, optional
        """
        if document_type is not SENTINEL:
            self.document_type = document_type
        if process_id is not SENTINEL:
            self.process_id = process_id
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .observability_endpoint import ObservabilityEndpoint


@JsonMap({"observability_endpoint": "observabilityEndpoint"})
class GeneralSettings(BaseModel):
    """GeneralSettings

    :param observability_endpoint: observability_endpoint
    :type observability_endpoint: ObservabilityEndpoint
    :param enabled: enabled, defaults to None
    :type enabled: bool, optional
    """

    def __init__(
        self,
        observability_endpoint: ObservabilityEndpoint,
        enabled: bool = SENTINEL,
        **kwargs,
    ):
        """GeneralSettings

        :param observability_endpoint: observability_endpoint
        :type observability_endpoint: ObservabilityEndpoint
        :param enabled: enabled, defaults to None
        :type enabled: bool, optional
        """
        self.observability_endpoint = self._define_object(
            observability_endpoint, ObservabilityEndpoint
        )
        if enabled is not SENTINEL:
            self.enabled = enabled
        self._kwargs = kwargs




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .execution_connector_simple_expression import ExecutionConnectorSimpleExpression
from .execution_connector_grouping_expression import (
    ExecutionConnectorGroupingExpression,
)


class ExecutionConnectorExpressionGuard(OneOfBaseModel):
    class_list = {
        "ExecutionConnectorSimpleExpression": ExecutionConnectorSimpleExpression,
        "ExecutionConnectorGroupingExpression": ExecutionConnectorGroupingExpression,
    }


ExecutionConnectorExpression = Union[
    ExecutionConnectorSimpleExpression, ExecutionConnectorGroupingExpression
]




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .x12_connector_record_simple_expression import X12ConnectorRecordSimpleExpression
from .x12_connector_record_grouping_expression import (
    X12ConnectorRecordGroupingExpression,
)


class X12ConnectorRecordExpressionGuard(OneOfBaseModel):
    class_list = {
        "X12ConnectorRecordSimpleExpression": X12ConnectorRecordSimpleExpression,
        "X12ConnectorRecordGroupingExpression": X12ConnectorRecordGroupingExpression,
    }


X12ConnectorRecordExpression = Union[
    X12ConnectorRecordSimpleExpression, X12ConnectorRecordGroupingExpression
]




from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class AtomSimpleExpressionOperator(Enum):
    """An enumeration representing different categories.

    :cvar EQUALS: "EQUALS"
    :vartype EQUALS: str
    :cvar NOTEQUALS: "NOT_EQUALS"
    :vartype NOTEQUALS: str
    :cvar CONTAINS: "CONTAINS"
    :vartype CONTAINS: str
    :cvar NOTCONTAINS: "NOT_CONTAINS"
    :vartype NOTCONTAINS: str
    """

    EQUALS = "EQUALS"
    NOTEQUALS = "NOT_EQUALS"
    CONTAINS = "CONTAINS"
    NOTCONTAINS = "NOT_CONTAINS"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, AtomSimpleExpressionOperator._member_map_.values())
        )


class AtomSimpleExpressionProperty(Enum):
    """An enumeration representing different categories.

    :cvar NAME: "name"
    :vartype NAME: str
    :cvar ID: "id"
    :vartype ID: str
    :cvar HOSTNAME: "hostname"
    :vartype HOSTNAME: str
    :cvar STATUS: "status"
    :vartype STATUS: str
    :cvar TYPE: "type"
    :vartype TYPE: str
    :cvar CAPABILITIES: "capabilities"
    :vartype CAPABILITIES: str
    """

    NAME = "name"
    ID = "id"
    HOSTNAME = "hostname"
    STATUS = "status"
    TYPE = "type"
    CAPABILITIES = "capabilities"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, AtomSimpleExpressionProperty._member_map_.values())
        )


@JsonMap({})
class AtomSimpleExpression(BaseModel):
    """AtomSimpleExpression

    :param argument: argument, defaults to None
    :type argument: List[str], optional
    :param operator: operator
    :type operator: AtomSimpleExpressionOperator
    :param property: property
    :type property: AtomSimpleExpressionProperty
    """

    def __init__(
        self,
        operator: AtomSimpleExpressionOperator,
        property: AtomSimpleExpressionProperty,
        argument: List[str] = SENTINEL,
        **kwargs
    ):
        """AtomSimpleExpression

        :param argument: argument, defaults to None
        :type argument: List[str], optional
        :param operator: operator
        :type operator: AtomSimpleExpressionOperator
        :param property: property
        :type property: AtomSimpleExpressionProperty
        """
        if argument is not SENTINEL:
            self.argument = argument
        self.operator = self._enum_matching(
            operator, AtomSimpleExpressionOperator.list(), "operator"
        )
        self.property = self._enum_matching(
            property, AtomSimpleExpressionProperty.list(), "property"
        )
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..environment import EnvironmentService
from ...models import (
    Environment,
    EnvironmentBulkResponse,
    EnvironmentBulkRequest,
    EnvironmentQueryResponse,
    EnvironmentQueryConfig,
    EnvironmentMapExtension,
)


class EnvironmentServiceAsync(EnvironmentService):
    """
    Async Wrapper for EnvironmentServiceAsync
    """

    def create_environment(
        self, request_body: Environment = None
    ) -> Awaitable[Union[Environment, str]]:
        return to_async(super().create_environment)(request_body)

    def get_environment(self, id_: str) -> Awaitable[Union[Environment, str]]:
        return to_async(super().get_environment)(id_)

    def update_environment(
        self, id_: str, request_body: Environment = None
    ) -> Awaitable[Union[Environment, str]]:
        return to_async(super().update_environment)(id_, request_body)

    def delete_environment(self, id_: str) -> Awaitable[None]:
        return to_async(super().delete_environment)(id_)

    def bulk_environment(
        self, request_body: EnvironmentBulkRequest = None
    ) -> Awaitable[Union[EnvironmentBulkResponse, str]]:
        return to_async(super().bulk_environment)(request_body)

    def query_environment(
        self, request_body: EnvironmentQueryConfig = None
    ) -> Awaitable[Union[EnvironmentQueryResponse, str]]:
        return to_async(super().query_environment)(request_body)

    def query_more_environment(
        self, request_body: str
    ) -> Awaitable[Union[EnvironmentQueryResponse, str]]:
        return to_async(super().query_more_environment)(request_body)

    def update_environment_map_extension(
        self, id_: str, request_body: EnvironmentMapExtension = None
    ) -> Awaitable[Union[EnvironmentMapExtension, str]]:
        return to_async(super().update_environment_map_extension)(id_, request_body)




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .execution_request_dynamic_process_properties import (
    ExecutionRequestDynamicProcessProperties,
)
from .execution_request_process_properties import ExecutionRequestProcessProperties


@JsonMap(
    {
        "dynamic_process_properties": "DynamicProcessProperties",
        "process_properties": "ProcessProperties",
        "atom_id": "atomId",
        "process_id": "processId",
        "process_name": "processName",
        "record_url": "recordUrl",
        "request_id": "requestId",
    }
)
class ExecutionRequest(BaseModel):
    """ExecutionRequest

    :param dynamic_process_properties: The full list of Dynamic Process properties within the specified Runtime, Runtime cluster, or cloud, where each property is defined by their name and value.
    :type dynamic_process_properties: ExecutionRequestDynamicProcessProperties
    :param process_properties: The full list of Process properties within the specified Runtime, Runtime cluster, or cloud, where each property is defined by their name and value.
    :type process_properties: ExecutionRequestProcessProperties
    :param atom_id: The ID of the Runtime on which to run the process. Locate the Runtime ID by navigating to **Manage** \\> **Runtime Management** on the user interface, and viewing the Runtime Information panel for a selected Runtime.
    :type atom_id: str
    :param process_id: The ID of the process to run. You can find ID of a process by locating the process' **Component ID** in the **Revision History** dialog on the user interface., defaults to None
    :type process_id: str, optional
    :param process_name: process_name, defaults to None
    :type process_name: str, optional
    :param record_url: \(Response-only field\) The ID of the process run. This field is returned in the initial POST response and is used in the subsequent call to find the corresponding run record., defaults to None
    :type record_url: str, optional
    :param request_id: request_id, defaults to None
    :type request_id: str, optional
    """

    def __init__(
        self,
        dynamic_process_properties: ExecutionRequestDynamicProcessProperties = SENTINEL,
        process_properties: ExecutionRequestProcessProperties = SENTINEL,
        atom_id: str = SENTINEL,
        process_id: str = SENTINEL,
        process_name: str = SENTINEL,
        record_url: str = SENTINEL,
        request_id: str = SENTINEL,
        **kwargs,
    ):
        """ExecutionRequest

        :param dynamic_process_properties: The full list of Dynamic Process properties within the specified Runtime, Runtime cluster, or cloud, where each property is defined by their name and value., defaults to None
        :type dynamic_process_properties: ExecutionRequestDynamicProcessProperties, optional
        :param process_properties: The full list of Process properties within the specified Runtime, Runtime cluster, or cloud, where each property is defined by their name and value., defaults to None
        :type process_properties: ExecutionRequestProcessProperties, optional
        :param atom_id: The ID of the Runtime on which to run the process. Locate the Runtime ID by navigating to **Manage** \\> **Runtime Management** on the user interface, and viewing the Runtime Information panel for a selected Runtime., defaults to None
        :type atom_id: str, optional
        :param process_id: The ID of the process to run. You can find ID of a process by locating the process' **Component ID** in the **Revision History** dialog on the user interface., defaults to None
        :type process_id: str, optional
        :param process_name: process_name, defaults to None
        :type process_name: str, optional
        :param record_url: \(Response-only field\) The ID of the process run. This field is returned in the initial POST response and is used in the subsequent call to find the corresponding run record., defaults to None
        :type record_url: str, optional
        :param request_id: request_id, defaults to None
        :type request_id: str, optional
        """
        if atom_id is not SENTINEL:
            self.atom_id = atom_id
        if dynamic_process_properties is not SENTINEL:
            self.dynamic_process_properties = self._define_object(
                dynamic_process_properties, ExecutionRequestDynamicProcessProperties
            )
        if process_properties is not SENTINEL:
            self.process_properties = self._define_object(
                process_properties, ExecutionRequestProcessProperties
            )
        if process_id is not SENTINEL:
            self.process_id = process_id
        if process_name is not SENTINEL:
            self.process_name = process_name
        if record_url is not SENTINEL:
            self.record_url = record_url
        if request_id is not SENTINEL:
            self.request_id = request_id
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..component_reference import ComponentReferenceService
from ...models import (
    ComponentReference,
    ComponentReferenceBulkResponse,
    ComponentReferenceBulkRequest,
    ComponentReferenceQueryResponse,
    ComponentReferenceQueryConfig,
)


class ComponentReferenceServiceAsync(ComponentReferenceService):
    """
    Async Wrapper for ComponentReferenceServiceAsync
    """

    def get_component_reference(
        self, component_id: str
    ) -> Awaitable[Union[ComponentReference, str]]:
        return to_async(super().get_component_reference)(component_id)

    def bulk_component_reference(
        self, request_body: ComponentReferenceBulkRequest = None
    ) -> Awaitable[Union[ComponentReferenceBulkResponse, str]]:
        return to_async(super().bulk_component_reference)(request_body)

    def query_component_reference(
        self, request_body: ComponentReferenceQueryConfig = None
    ) -> Awaitable[Union[ComponentReferenceQueryResponse, str]]:
        return to_async(super().query_component_reference)(request_body)

    def query_more_component_reference(
        self, request_body: str
    ) -> Awaitable[Union[ComponentReferenceQueryResponse, str]]:
        return to_async(super().query_more_component_reference)(request_body)




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({"id_": "id"})
class BulkId(BaseModel):
    """BulkId

    :param id_: id_, defaults to None
    :type id_: str, optional
    """

    def __init__(self, id_: str = SENTINEL, **kwargs):
        """BulkId

        :param id_: id_, defaults to None
        :type id_: str, optional
        """
        if id_ is not SENTINEL:
            self.id_ = id_
        self._kwargs = kwargs




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import (
    Environment,
    EnvironmentBulkRequest,
    EnvironmentBulkResponse,
    EnvironmentMapExtension,
    EnvironmentQueryConfig,
    EnvironmentQueryResponse,
)


class EnvironmentService(BaseService):

    @cast_models
    def create_environment(
        self, request_body: Environment = None
    ) -> Union[Environment, str]:
        """Creates an environment having the specified name. Environment names must be unique.

        :param request_body: The request body., defaults to None
        :type request_body: Environment, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[Environment, str]
        """

        Validator(Environment).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Environment",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return Environment._unmap(response)
        if content == "application/xml":
            return Environment._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def get_environment(self, id_: str) -> Union[Environment, str]:
        """Retrieves the properties of the environment with a specified ID.

        :param id_: A unique ID assigned by the system to the environment.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[Environment, str]
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Environment/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return Environment._unmap(response)
        if content == "application/xml":
            return Environment._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def update_environment(
        self, id_: str, request_body: Environment = None
    ) -> Union[Environment, str]:
        """Updates the Environment object having the specified ID. You can edit the name field only.

        :param request_body: The request body., defaults to None
        :type request_body: Environment, optional
        :param id_: A unique ID assigned by the system to the environment.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[Environment, str]
        """

        Validator(Environment).is_optional().validate(request_body)
        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Environment/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return Environment._unmap(response)
        if content == "application/xml":
            return Environment._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def delete_environment(self, id_: str) -> None:
        """Deletes the Environment object with a specified ID. It is not possible to delete an environment that has attached Runtimes or integration packs.

        :param id_: A unique ID assigned by the system to the environment.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Environment/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("DELETE")
        )

        response, status, _ = self.send_request(serialized_request)

    @cast_models
    def bulk_environment(
        self, request_body: EnvironmentBulkRequest = None
    ) -> Union[EnvironmentBulkResponse, str]:
        """To learn more about `bulk`, refer to [Bulk GET operations](#section/Introduction/Bulk-GET-operations).

        :param request_body: The request body., defaults to None
        :type request_body: EnvironmentBulkRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[EnvironmentBulkResponse, str]
        """

        Validator(EnvironmentBulkRequest).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Environment/bulk",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return EnvironmentBulkResponse._unmap(response)
        if content == "application/xml":
            return EnvironmentBulkResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_environment(
        self, request_body: EnvironmentQueryConfig = None
    ) -> Union[EnvironmentQueryResponse, str]:
        """For general information about the structure of QUERY filters, their sample payloads, and how to handle the paged results, refer to [Query filters](#section/Introduction/Query-filters) and [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body., defaults to None
        :type request_body: EnvironmentQueryConfig, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[EnvironmentQueryResponse, str]
        """

        Validator(EnvironmentQueryConfig).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Environment/query",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return EnvironmentQueryResponse._unmap(response)
        if content == "application/xml":
            return EnvironmentQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_more_environment(
        self, request_body: str
    ) -> Union[EnvironmentQueryResponse, str]:
        """To learn about using `queryMore`, refer to [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body.
        :type request_body: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[EnvironmentQueryResponse, str]
        """

        Validator(str).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/Environment/queryMore",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body, "text/plain")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return EnvironmentQueryResponse._unmap(response)
        if content == "application/xml":
            return EnvironmentQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def update_environment_map_extension(
        self, id_: str, request_body: EnvironmentMapExtension = None
    ) -> Union[EnvironmentMapExtension, str]:
        """Updates the extended mapping configuration for the specified Environment Map Extension object ID.

        :param request_body: The request body., defaults to None
        :type request_body: EnvironmentMapExtension, optional
        :param id_: id_
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[EnvironmentMapExtension, str]
        """

        Validator(EnvironmentMapExtension).is_optional().validate(request_body)
        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/EnvironmentMapExtension/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return EnvironmentMapExtension._unmap(response)
        if content == "application/xml":
            return EnvironmentMapExtension._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .unb_control_info import UnbControlInfo
from .ung_control_info import UngControlInfo
from .unh_control_info import UnhControlInfo


@JsonMap(
    {
        "unb_control_info": "UNBControlInfo",
        "ung_control_info": "UNGControlInfo",
        "unh_control_info": "UNHControlInfo",
    }
)
class EdifactControlInfo(BaseModel):
    """EdifactControlInfo

    :param unb_control_info: unb_control_info, defaults to None
    :type unb_control_info: UnbControlInfo, optional
    :param ung_control_info: ung_control_info, defaults to None
    :type ung_control_info: UngControlInfo, optional
    :param unh_control_info: unh_control_info, defaults to None
    :type unh_control_info: UnhControlInfo, optional
    """

    def __init__(
        self,
        unb_control_info: UnbControlInfo = SENTINEL,
        ung_control_info: UngControlInfo = SENTINEL,
        unh_control_info: UnhControlInfo = SENTINEL,
        **kwargs,
    ):
        """EdifactControlInfo

        :param unb_control_info: unb_control_info, defaults to None
        :type unb_control_info: UnbControlInfo, optional
        :param ung_control_info: ung_control_info, defaults to None
        :type ung_control_info: UngControlInfo, optional
        :param unh_control_info: unh_control_info, defaults to None
        :type unh_control_info: UnhControlInfo, optional
        """
        if unb_control_info is not SENTINEL:
            self.unb_control_info = self._define_object(unb_control_info, UnbControlInfo)
        if ung_control_info is not SENTINEL:
            self.ung_control_info = self._define_object(
                ung_control_info, UngControlInfo
            )
        if unh_control_info is not SENTINEL:
            self.unh_control_info = self._define_object(unh_control_info, UnhControlInfo)
        self._kwargs = kwargs




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import MoveQueueRequest


class MoveQueueRequestService(BaseService):

    @cast_models
    def create_move_queue_request(
        self, request_body: MoveQueueRequest = None
    ) -> Union[MoveQueueRequest, str]:
        """Moves messages from one Runtime queue to another.

         - You must have the **Runtime Management** privilege to use the Move queue request operation.
        - You can only move messages between queues of the same type. For example, moving queue messages from a point-to-point queue to a *Publish/Subscribe* queue results in an error.
        - Any Runtime queues that you specify in the request must currently exist on the Runtime. For clarification, you cannot create a new endpoint using the CREATE operation.
        - You must supply the *AtomID*, *SourceQueue*, *QueueName*, *DLQ*, and *DestinationQueue* fields in the request.
        - If the operation returns an error, it logs a message in the Runtime, Runtime cluster, or Runtime cloud, but is not sent to the platform.
        - You cannot track move updates directly through this operation. Instead, to see if the action is in progress or complete, use the Queue Management API or the ListQueues API to observe the number of messages in the queue.

        :param request_body: The request body., defaults to None
        :type request_body: MoveQueueRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[MoveQueueRequest, str]
        """

        Validator(MoveQueueRequest).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/MoveQueueRequest",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return MoveQueueRequest._unmap(response)
        if content == "application/xml":
            return MoveQueueRequest._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..hl7_connector_record import Hl7ConnectorRecordService
from ...models import Hl7ConnectorRecordQueryResponse, Hl7ConnectorRecordQueryConfig


class Hl7ConnectorRecordServiceAsync(Hl7ConnectorRecordService):
    """
    Async Wrapper for Hl7ConnectorRecordServiceAsync
    """

    def query_hl7_connector_record(
        self, request_body: Hl7ConnectorRecordQueryConfig = None
    ) -> Awaitable[Union[Hl7ConnectorRecordQueryResponse, str]]:
        return to_async(super().query_hl7_connector_record)(request_body)

    def query_more_hl7_connector_record(
        self, request_body: str
    ) -> Awaitable[Union[Hl7ConnectorRecordQueryResponse, str]]:
        return to_async(super().query_more_hl7_connector_record)(request_body)




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .as2_mdn_options import As2MdnOptions
from .as2_message_options import As2MessageOptions
from .as2_partner_info import As2PartnerInfo


@JsonMap(
    {
        "as2_mdn_options": "AS2MDNOptions",
        "as2_message_options": "AS2MessageOptions",
        "as2_partner_info": "AS2PartnerInfo",
    }
)
class As2SendOptions(BaseModel):
    """As2SendOptions

    :param as2_mdn_options: as2_mdn_options
    :type as2_mdn_options: As2MdnOptions
    :param as2_message_options: as2_message_options
    :type as2_message_options: As2MessageOptions
    :param as2_partner_info: as2_partner_info, defaults to None
    :type as2_partner_info: As2PartnerInfo, optional
    """

    def __init__(
        self,
        as2_mdn_options: As2MdnOptions,
        as2_message_options: As2MessageOptions,
        as2_partner_info: As2PartnerInfo = SENTINEL,
        **kwargs,
    ):
        """As2SendOptions

        :param as2_mdn_options: as2_mdn_options
        :type as2_mdn_options: As2MdnOptions
        :param as2_message_options: as2_message_options
        :type as2_message_options: As2MessageOptions
        :param as2_partner_info: as2_partner_info, defaults to None
        :type as2_partner_info: As2PartnerInfo, optional
        """
        self.as2_mdn_options = self._define_object(as2_mdn_options, As2MdnOptions)
        self.as2_message_options = self._define_object(
            as2_message_options, As2MessageOptions
        )
        if as2_partner_info is not SENTINEL:
            self.as2_partner_info = self._define_object(
                as2_partner_info, As2PartnerInfo
            )
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({"created_at": "createdAt", "updated_at": "updatedAt"})
class AccountGroupIntegrationPackExpressionMetadata(BaseModel):
    """AccountGroupIntegrationPackExpressionMetadata

    :param created_at: created_at, defaults to None
    :type created_at: str, optional
    :param updated_at: updated_at, defaults to None
    :type updated_at: str, optional
    """

    def __init__(
        self, created_at: str = SENTINEL, updated_at: str = SENTINEL, **kwargs
    ):
        """AccountGroupIntegrationPackExpressionMetadata

        :param created_at: created_at, defaults to None
        :type created_at: str, optional
        :param updated_at: updated_at, defaults to None
        :type updated_at: str, optional
        """
        if created_at is not SENTINEL:
            self.created_at = created_at
        if updated_at is not SENTINEL:
            self.updated_at = updated_at
        self._kwargs = kwargs


@JsonMap({"id_": "id"})
class AccountGroupIntegrationPackExpression(BaseModel):
    """AccountGroupIntegrationPackExpression

    :param id_: A unique ID assigned by the system to the integration pack. This field populates only if you add the integration pack to an account group., defaults to None
    :type id_: str, optional
    :param name: The name of the integration pack., defaults to None
    :type name: str, optional
    :param status: status, defaults to None
    :type status: str, optional
    :param metadata: metadata, defaults to None
    :type metadata: AccountGroupIntegrationPackExpressionMetadata, optional
    """

    def __init__(
        self,
        id_: str = SENTINEL,
        name: str = SENTINEL,
        status: str = SENTINEL,
        metadata: AccountGroupIntegrationPackExpressionMetadata = SENTINEL,
        **kwargs
    ):
        """AccountGroupIntegrationPackExpression

        :param id_: A unique ID assigned by the system to the integration pack. This field populates only if you add the integration pack to an account group., defaults to None
        :type id_: str, optional
        :param name: The name of the integration pack., defaults to None
        :type name: str, optional
        :param status: status, defaults to None
        :type status: str, optional
        :param metadata: metadata, defaults to None
        :type metadata: AccountGroupIntegrationPackExpressionMetadata, optional
        """
        if id_ is not SENTINEL:
            self.id_ = id_
        if name is not SENTINEL:
            self.name = name
        if status is not SENTINEL:
            self.status = status
        if metadata is not SENTINEL:
            self.metadata = self._define_object(
                metadata, AccountGroupIntegrationPackExpressionMetadata
            )
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..throughput_account_group import ThroughputAccountGroupService
from ...models import (
    ThroughputAccountGroupQueryResponse,
    ThroughputAccountGroupQueryConfig,
)


class ThroughputAccountGroupServiceAsync(ThroughputAccountGroupService):
    """
    Async Wrapper for ThroughputAccountGroupServiceAsync
    """

    def query_throughput_account_group(
        self, request_body: ThroughputAccountGroupQueryConfig = None
    ) -> Awaitable[Union[ThroughputAccountGroupQueryResponse, str]]:
        return to_async(super().query_throughput_account_group)(request_body)

    def query_more_throughput_account_group(
        self, request_body: str
    ) -> Awaitable[Union[ThroughputAccountGroupQueryResponse, str]]:
        return to_async(super().query_more_throughput_account_group)(request_body)




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..connector_document import ConnectorDocumentService
from ...models import ConnectorDocumentDownload, ConnectorDocument


class ConnectorDocumentServiceAsync(ConnectorDocumentService):
    """
    Async Wrapper for ConnectorDocumentServiceAsync
    """

    def create_connector_document(
        self, request_body: ConnectorDocument = None
    ) -> Awaitable[Union[ConnectorDocumentDownload, str]]:
        return to_async(super().create_connector_document)(request_body)

    def download_connector_document(
        self,
        request_body: ConnectorDocument = None,
        max_retries: int = 10,
        initial_delay: float = 2.0,
    ) -> Awaitable[bytes]:
        return to_async(super().download_connector_document)(
            request_body, max_retries, initial_delay
        )




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import (
    AccountCloudAttachmentSummary,
    AccountCloudAttachmentSummaryBulkRequest,
    AccountCloudAttachmentSummaryBulkResponse,
    AccountCloudAttachmentSummaryQueryConfig,
    AccountCloudAttachmentSummaryQueryResponse,
)


class AccountCloudAttachmentSummaryService(BaseService):
    """Service for AccountCloudAttachmentSummary endpoints.

    .. note::
        The Cloud Management feature must be enabled for your Boomi account.
        Without it, all AccountCloudAttachmentSummary endpoints return HTTP 400
        with the message "The Cloud Management feature is not enabled for your
        account."  This response does not indicate a malformed SDK request.
    """

    @cast_models
    def get_account_cloud_attachment_summary(
        self, id_: str
    ) -> Union[AccountCloudAttachmentSummary, str]:
        """Retrieves the properties of the AccountCloudAttachmentSummary having the specified ID.

        :param id_: A unique ID assigned by the system.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AccountCloudAttachmentSummary, str]
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/AccountCloudAttachmentSummary/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AccountCloudAttachmentSummary._unmap(response)
        if content == "application/xml":
            return AccountCloudAttachmentSummary._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def bulk_account_cloud_attachment_summary(
        self, request_body: AccountCloudAttachmentSummaryBulkRequest = None
    ) -> Union[AccountCloudAttachmentSummaryBulkResponse, str]:
        """To learn more about `bulk`, refer to [Bulk GET operations](#section/Introduction/Bulk-GET-operations).

        :param request_body: The request body., defaults to None
        :type request_body: AccountCloudAttachmentSummaryBulkRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AccountCloudAttachmentSummaryBulkResponse, str]
        """

        Validator(AccountCloudAttachmentSummaryBulkRequest).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/AccountCloudAttachmentSummary/bulk",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AccountCloudAttachmentSummaryBulkResponse._unmap(response)
        if content == "application/xml":
            return AccountCloudAttachmentSummaryBulkResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_account_cloud_attachment_summary(
        self, request_body: AccountCloudAttachmentSummaryQueryConfig = None
    ) -> Union[AccountCloudAttachmentSummaryQueryResponse, str]:
        """For general information about the structure of QUERY filters, their sample payloads, and how to handle the paged results, refer to [Query filters](#section/Introduction/Query-filters) and [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body., defaults to None
        :type request_body: AccountCloudAttachmentSummaryQueryConfig, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AccountCloudAttachmentSummaryQueryResponse, str]
        """

        Validator(AccountCloudAttachmentSummaryQueryConfig).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/AccountCloudAttachmentSummary/query",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AccountCloudAttachmentSummaryQueryResponse._unmap(response)
        if content == "application/xml":
            return AccountCloudAttachmentSummaryQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_more_account_cloud_attachment_summary(
        self, request_body: str
    ) -> Union[AccountCloudAttachmentSummaryQueryResponse, str]:
        """To learn about using `queryMore`, refer to [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body.
        :type request_body: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AccountCloudAttachmentSummaryQueryResponse, str]
        """

        Validator(str).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/AccountCloudAttachmentSummary/queryMore",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body, "text/plain")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AccountCloudAttachmentSummaryQueryResponse._unmap(response)
        if content == "application/xml":
            return AccountCloudAttachmentSummaryQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap(
    {
        "interchange_id": "interchangeId",
        "interchange_id_qualifier": "interchangeIdQualifier",
    }
)
class StxControlInfo(BaseModel):
    """StxControlInfo

    :param interchange_id: interchange_id, defaults to None
    :type interchange_id: str, optional
    :param interchange_id_qualifier: interchange_id_qualifier, defaults to None
    :type interchange_id_qualifier: str, optional
    """

    def __init__(
        self,
        interchange_id: str = SENTINEL,
        interchange_id_qualifier: str = SENTINEL,
        **kwargs
    ):
        """StxControlInfo

        :param interchange_id: interchange_id, defaults to None
        :type interchange_id: str, optional
        :param interchange_id_qualifier: interchange_id_qualifier, defaults to None
        :type interchange_id_qualifier: str, optional
        """
        if interchange_id is not SENTINEL:
            self.interchange_id = interchange_id
        if interchange_id_qualifier is not SENTINEL:
            self.interchange_id_qualifier = interchange_id_qualifier
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..integration_pack import IntegrationPackService
from ...models import (
    IntegrationPack,
    IntegrationPackBulkResponse,
    IntegrationPackBulkRequest,
    IntegrationPackQueryResponse,
    IntegrationPackQueryConfig,
)


class IntegrationPackServiceAsync(IntegrationPackService):
    """
    Async Wrapper for IntegrationPackServiceAsync
    """

    def get_integration_pack(self, id_: str) -> Awaitable[Union[IntegrationPack, str]]:
        return to_async(super().get_integration_pack)(id_)

    def bulk_integration_pack(
        self, request_body: IntegrationPackBulkRequest = None
    ) -> Awaitable[Union[IntegrationPackBulkResponse, str]]:
        return to_async(super().bulk_integration_pack)(request_body)

    def query_integration_pack(
        self, request_body: IntegrationPackQueryConfig = None
    ) -> Awaitable[Union[IntegrationPackQueryResponse, str]]:
        return to_async(super().query_integration_pack)(request_body)

    def query_more_integration_pack(
        self, request_body: str
    ) -> Awaitable[Union[IntegrationPackQueryResponse, str]]:
        return to_async(super().query_more_integration_pack)(request_body)




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .override_process_property import OverrideProcessProperty


@JsonMap({"process_property": "ProcessProperty"})
class OverrideProcessProperties(BaseModel):
    """OverrideProcessProperties

    :param process_property: process_property, defaults to None
    :type process_property: List[OverrideProcessProperty], optional
    """

    def __init__(
        self, process_property: List[OverrideProcessProperty] = SENTINEL, **kwargs
    ):
        """OverrideProcessProperties

        :param process_property: process_property, defaults to None
        :type process_property: List[OverrideProcessProperty], optional
        """
        if process_property is not SENTINEL:
            self.process_property = self._define_list(
                process_property, OverrideProcessProperty
            )
        self._kwargs = kwargs




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .account_simple_expression import AccountSimpleExpression
from .account_grouping_expression import AccountGroupingExpression


class AccountExpressionGuard(OneOfBaseModel):
    class_list = {
        "AccountSimpleExpression": AccountSimpleExpression,
        "AccountGroupingExpression": AccountGroupingExpression,
    }


AccountExpression = Union[AccountSimpleExpression, AccountGroupingExpression]




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..organization_component import OrganizationComponentService
from ...models import (
    OrganizationComponent,
    OrganizationComponentBulkResponse,
    OrganizationComponentBulkRequest,
    OrganizationComponentQueryResponse,
    OrganizationComponentQueryConfig,
)


class OrganizationComponentServiceAsync(OrganizationComponentService):
    """
    Async Wrapper for OrganizationComponentServiceAsync
    """

    def create_organization_component(
        self, request_body: OrganizationComponent = None
    ) -> Awaitable[Union[OrganizationComponent, str]]:
        return to_async(super().create_organization_component)(request_body)

    def get_organization_component(
        self, id_: str
    ) -> Awaitable[Union[OrganizationComponent, str]]:
        return to_async(super().get_organization_component)(id_)

    def update_organization_component(
        self, id_: str, request_body: OrganizationComponent = None
    ) -> Awaitable[Union[OrganizationComponent, str]]:
        return to_async(super().update_organization_component)(id_, request_body)

    def delete_organization_component(self, id_: str) -> Awaitable[None]:
        return to_async(super().delete_organization_component)(id_)

    def bulk_organization_component(
        self, request_body: OrganizationComponentBulkRequest = None
    ) -> Awaitable[Union[str, OrganizationComponentBulkResponse]]:
        return to_async(super().bulk_organization_component)(request_body)

    def query_organization_component(
        self, request_body: OrganizationComponentQueryConfig = None
    ) -> Awaitable[Union[OrganizationComponentQueryResponse, str]]:
        return to_async(super().query_organization_component)(request_body)

    def query_more_organization_component(
        self, request_body: str
    ) -> Awaitable[Union[OrganizationComponentQueryResponse, str]]:
        return to_async(super().query_more_organization_component)(request_body)




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap(
    {
        "header_field_name": "headerFieldName",
        "target_property_name": "targetPropertyName",
    }
)
class Header(BaseModel):
    """Header

    :param header_field_name: header_field_name, defaults to None
    :type header_field_name: str, optional
    :param target_property_name: target_property_name, defaults to None
    :type target_property_name: str, optional
    """

    def __init__(
        self,
        header_field_name: str = SENTINEL,
        target_property_name: str = SENTINEL,
        **kwargs
    ):
        """Header

        :param header_field_name: header_field_name, defaults to None
        :type header_field_name: str, optional
        :param target_property_name: target_property_name, defaults to None
        :type target_property_name: str, optional
        """
        if header_field_name is not SENTINEL:
            self.header_field_name = header_field_name
        if target_property_name is not SENTINEL:
            self.target_property_name = target_property_name
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({})
class HttpAuthSettings(BaseModel):
    """HttpAuthSettings

    :param password: password, defaults to None
    :type password: str, optional
    :param user: user, defaults to None
    :type user: str, optional
    """

    def __init__(self, password: str = SENTINEL, user: str = SENTINEL, **kwargs):
        """HttpAuthSettings

        :param password: password, defaults to None
        :type password: str, optional
        :param user: user, defaults to None
        :type user: str, optional
        """
        if password is not SENTINEL:
            self.password = password
        if user is not SENTINEL:
            self.user = user
        self._kwargs = kwargs




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .process_environment_attachment_simple_expression import (
    ProcessEnvironmentAttachmentSimpleExpression,
)
from .process_environment_attachment_grouping_expression import (
    ProcessEnvironmentAttachmentGroupingExpression,
)


class ProcessEnvironmentAttachmentExpressionGuard(OneOfBaseModel):
    class_list = {
        "ProcessEnvironmentAttachmentSimpleExpression": ProcessEnvironmentAttachmentSimpleExpression,
        "ProcessEnvironmentAttachmentGroupingExpression": ProcessEnvironmentAttachmentGroupingExpression,
    }


ProcessEnvironmentAttachmentExpression = Union[
    ProcessEnvironmentAttachmentSimpleExpression,
    ProcessEnvironmentAttachmentGroupingExpression,
]




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import (
    PackagedComponent,
    PackagedComponentBulkRequest,
    PackagedComponentBulkResponse,
    PackagedComponentQueryConfig,
    PackagedComponentQueryResponse,
)


class PackagedComponentService(BaseService):

    @cast_models
    def create_packaged_component(
        self, request_body: PackagedComponent = None
    ) -> Union[PackagedComponent, str]:
        """- You can use the CREATE operation to perform two different actions. For example, you can create a new packaged component from a specific component ID, or you can restore a deleted packaged component. Both actions use the same object endpoint. However, the information required in the request body differs.
          -  **To create a new packaged component**, you must include a component ID in the request body. You create a packaged component for the specified componentId. Optionally, you can specify a packageVersion value and notes about the package version.
             >**Note:** You cannot add package versions and notes after creating the packaged component. However, if not specified, automatically assigns a numerical version number to your new packaged component.
          -  **To restore or recover a deleted packaged component**, you must specify the packageId, componentId, and packageVersion. You can query the Packaged Component object for a list of deleted packaged components.
         - Specify a `branchName` to create a packaged component on a particular branch. If `branchName` is not provided, the default working branch is used.

        :param request_body: The request body., defaults to None
        :type request_body: PackagedComponent, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[PackagedComponent, str]
        """

        Validator(PackagedComponent).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/PackagedComponent",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return PackagedComponent._unmap(response)
        if content == "application/xml":
            return PackagedComponent._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def get_packaged_component(self, id_: str) -> Union[PackagedComponent, str]:
        """Retrieves the packaged component with the specified ID.

        :param id_: id_
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[PackagedComponent, str]
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/PackagedComponent/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return PackagedComponent._unmap(response)
        if content == "application/xml":
            return PackagedComponent._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def delete_packaged_component(self, id_: str) -> None:
        """- The DELETE operation deletes a specific packaged component version. The id that you provide in the endpoint represents a Packaged Component ID. You can retrieve the Packaged Component ID (packageId) using the GET and QUERY operations, or by viewing the **Packaged Component History** dialog for a specific version in the Integration user interface.
          >**Note:** You can restore deleted packaged components using the CREATE operation. See the section **Using the CREATE operation** for more details.

        - You cannot delete a packaged component if it is already in use. If currently deployed, a packaged component is considered in use if it is used in the **Process Library** or as part of an integration pack.

        :param id_: id_
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/PackagedComponent/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("DELETE")
        )

        response, status, _ = self.send_request(serialized_request)

    @cast_models
    def bulk_packaged_component(
        self, request_body: PackagedComponentBulkRequest = None
    ) -> Union[PackagedComponentBulkResponse, str]:
        """To learn more about `bulk`, refer to [Bulk GET operations](#section/Introduction/Bulk-GET-operations).

        :param request_body: The request body., defaults to None
        :type request_body: PackagedComponentBulkRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[PackagedComponentBulkResponse, str]
        """

        Validator(PackagedComponentBulkRequest).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/PackagedComponent/bulk",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return PackagedComponentBulkResponse._unmap(response)
        if content == "application/xml":
            return PackagedComponentBulkResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_packaged_component(
        self, request_body: PackagedComponentQueryConfig = None
    ) -> Union[PackagedComponentQueryResponse, str]:
        """For general information about the structure of QUERY filters, their sample payloads, and how to handle the paged results, refer to [Query filters](#section/Introduction/Query-filters) and [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body., defaults to None
        :type request_body: PackagedComponentQueryConfig, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[PackagedComponentQueryResponse, str]
        """

        Validator(PackagedComponentQueryConfig).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/PackagedComponent/query",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return PackagedComponentQueryResponse._unmap(response)
        if content == "application/xml":
            return PackagedComponentQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_more_packaged_component(
        self, request_body: str
    ) -> Union[PackagedComponentQueryResponse, str]:
        """To learn about using `queryMore`, refer to [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body.
        :type request_body: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[PackagedComponentQueryResponse, str]
        """

        Validator(str).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/PackagedComponent/queryMore",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body, "text/plain")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return PackagedComponentQueryResponse._unmap(response)
        if content == "application/xml":
            return PackagedComponentQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .default_oftp_partner_send_settings import DefaultOftpPartnerSendSettings


@JsonMap({"default_partner_settings": "defaultPartnerSettings"})
class OftpSendOptionsInfo(BaseModel):
    """OftpSendOptionsInfo

    :param cd: cd, defaults to None
    :type cd: bool, optional
    :param default_partner_settings: default_partner_settings
    :type default_partner_settings: DefaultOftpPartnerSendSettings
    :param operation: operation, defaults to None
    :type operation: str, optional
    :param sfiddesc: sfiddesc, defaults to None
    :type sfiddesc: str, optional
    :param sfiddsn: sfiddsn, defaults to None
    :type sfiddsn: str, optional
    """

    def __init__(
        self,
        default_partner_settings: DefaultOftpPartnerSendSettings,
        cd: bool = SENTINEL,
        operation: str = SENTINEL,
        sfiddesc: str = SENTINEL,
        sfiddsn: str = SENTINEL,
        **kwargs,
    ):
        """OftpSendOptionsInfo

        :param cd: cd, defaults to None
        :type cd: bool, optional
        :param default_partner_settings: default_partner_settings
        :type default_partner_settings: DefaultOftpPartnerSendSettings
        :param operation: operation, defaults to None
        :type operation: str, optional
        :param sfiddesc: sfiddesc, defaults to None
        :type sfiddesc: str, optional
        :param sfiddsn: sfiddsn, defaults to None
        :type sfiddsn: str, optional
        """
        if cd is not SENTINEL:
            self.cd = cd
        self.default_partner_settings = self._define_object(
            default_partner_settings, DefaultOftpPartnerSendSettings
        )
        if operation is not SENTINEL:
            self.operation = operation
        if sfiddesc is not SENTINEL:
            self.sfiddesc = sfiddesc
        if sfiddsn is not SENTINEL:
            self.sfiddsn = sfiddsn
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap(
    {
        "atom_id": "atomId",
        "id_": "id",
        "integration_pack_instance_id": "integrationPackInstanceId",
    }
)
class IntegrationPackAtomAttachment(BaseModel):
    """IntegrationPackAtomAttachment

    :param atom_id: A unique ID assigned by the system to the Runtime., defaults to None
    :type atom_id: str, optional
    :param id_: The object’s conceptual ID, which is synthesized from the Runtime and integration pack instance IDs., defaults to None
    :type id_: str, optional
    :param integration_pack_instance_id: A unique ID assigned by the system to the integration pack instance., defaults to None
    :type integration_pack_instance_id: str, optional
    """

    def __init__(
        self,
        atom_id: str = SENTINEL,
        id_: str = SENTINEL,
        integration_pack_instance_id: str = SENTINEL,
        **kwargs
    ):
        """IntegrationPackAtomAttachment

        :param atom_id: A unique ID assigned by the system to the Runtime., defaults to None
        :type atom_id: str, optional
        :param id_: The object’s conceptual ID, which is synthesized from the Runtime and integration pack instance IDs., defaults to None
        :type id_: str, optional
        :param integration_pack_instance_id: A unique ID assigned by the system to the integration pack instance., defaults to None
        :type integration_pack_instance_id: str, optional
        """
        if atom_id is not SENTINEL:
            self.atom_id = atom_id
        if id_ is not SENTINEL:
            self.id_ = id_
        if integration_pack_instance_id is not SENTINEL:
            self.integration_pack_instance_id = integration_pack_instance_id
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({"environment_id": "environmentId", "id_": "id", "role_id": "roleId"})
class EnvironmentRole(BaseModel):
    """EnvironmentRole

    :param environment_id: The environment ID., defaults to None
    :type environment_id: str, optional
    :param id_: The object’s conceptual ID, which is synthesized from the role and environment IDs., defaults to None
    :type id_: str, optional
    :param role_id: The ID of the role., defaults to None
    :type role_id: str, optional
    """

    def __init__(
        self,
        environment_id: str = SENTINEL,
        id_: str = SENTINEL,
        role_id: str = SENTINEL,
        **kwargs
    ):
        """EnvironmentRole

        :param environment_id: The environment ID., defaults to None
        :type environment_id: str, optional
        :param id_: The object’s conceptual ID, which is synthesized from the role and environment IDs., defaults to None
        :type id_: str, optional
        :param role_id: The ID of the role., defaults to None
        :type role_id: str, optional
        """
        if environment_id is not SENTINEL:
            self.environment_id = environment_id
        if id_ is not SENTINEL:
            self.id_ = id_
        if role_id is not SENTINEL:
            self.role_id = role_id
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .tradacoms_connector_record_expression import (
    TradacomsConnectorRecordExpression,
    TradacomsConnectorRecordExpressionGuard,
)
from .tradacoms_connector_record_simple_expression import (
    TradacomsConnectorRecordSimpleExpression,
)
from .tradacoms_connector_record_grouping_expression import (
    TradacomsConnectorRecordGroupingExpression,
)


@JsonMap({})
class TradacomsConnectorRecordQueryConfigQueryFilter(BaseModel):
    """TradacomsConnectorRecordQueryConfigQueryFilter

    :param expression: expression
    :type expression: TradacomsConnectorRecordExpression
    """

    def __init__(self, expression: TradacomsConnectorRecordExpression, **kwargs):
        """TradacomsConnectorRecordQueryConfigQueryFilter

        :param expression: expression
        :type expression: TradacomsConnectorRecordExpression
        """
        self.expression = TradacomsConnectorRecordExpressionGuard.return_one_of(
            expression
        )
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class TradacomsConnectorRecordQueryConfig(BaseModel):
    """TradacomsConnectorRecordQueryConfig

    :param query_filter: query_filter
    :type query_filter: TradacomsConnectorRecordQueryConfigQueryFilter
    """

    def __init__(
        self, query_filter: TradacomsConnectorRecordQueryConfigQueryFilter, **kwargs
    ):
        """TradacomsConnectorRecordQueryConfig

        :param query_filter: query_filter
        :type query_filter: TradacomsConnectorRecordQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, TradacomsConnectorRecordQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..branch import BranchService
from ...models import (
    Branch,
    BranchBulkResponse,
    BranchBulkRequest,
    BranchQueryResponse,
    BranchQueryConfig,
)


class BranchServiceAsync(BranchService):
    """
    Async Wrapper for BranchServiceAsync
    """

    def create_branch(
        self, request_body: Branch = None
    ) -> Awaitable[Union[Branch, str]]:
        return to_async(super().create_branch)(request_body)

    def get_branch(self, id_: str) -> Awaitable[Union[Branch, str]]:
        return to_async(super().get_branch)(id_)

    def update_branch(
        self, id_: str, request_body: Branch = None
    ) -> Awaitable[Union[Branch, str]]:
        return to_async(super().update_branch)(id_, request_body)

    def delete_branch(self, id_: str) -> Awaitable[None]:
        return to_async(super().delete_branch)(id_)

    def bulk_branch(
        self, request_body: BranchBulkRequest = None
    ) -> Awaitable[Union[BranchBulkResponse, str]]:
        return to_async(super().bulk_branch)(request_body)

    def query_branch(
        self, request_body: BranchQueryConfig = None
    ) -> Awaitable[Union[BranchQueryResponse, str]]:
        return to_async(super().query_branch)(request_body)

    def query_more_branch(
        self, request_body: str
    ) -> Awaitable[Union[BranchQueryResponse, str]]:
        return to_async(super().query_more_branch)(request_body)




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import (
    TradingPartnerComponent,
    TradingPartnerComponentBulkRequest,
    TradingPartnerComponentBulkResponse,
    TradingPartnerComponentQueryConfig,
    TradingPartnerComponentQueryResponse,
)


class TradingPartnerComponentService(BaseService):

    @cast_models
    def create_trading_partner_component(
        self, request_body: TradingPartnerComponent = None
    ) -> Union[TradingPartnerComponent, str]:
        """- This operation creates a Trading Partner Component object with a specified component name.
         - The request body requires the standard, classification, and componentName fields. If you omit the folderName field, you must use the folderId field — and vice versa. If you omit the componentID field and the IDs of any certificates you want to create, their values are assigned when you create the components. If you leave off the folderID field when creating a component, it assigns a value.
         - Includes the organizationId field only if the trading partner is to reference an Organization component, in which case the field value is the ID of the Organization component. A request specifying the organizationId field populates the ContactInformation fields with the data from the referenced Organization component.

        :param request_body: The request body., defaults to None
        :type request_body: TradingPartnerComponent, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[TradingPartnerComponent, str]
        """

        Validator(TradingPartnerComponent).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/TradingPartnerComponent",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return TradingPartnerComponent._unmap(response)
        if content == "application/xml":
            return TradingPartnerComponent._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def get_trading_partner_component(
        self, id_: str
    ) -> Union[TradingPartnerComponent, str]:
        """The ordinary GET operation returns a single Trading Partner Component object based on the supplied ID. A GET operation specifying the ID of a deleted Trading Partner component retrieves the component. In the component, the deleted field’s value is true.

        :param id_: id_
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[TradingPartnerComponent, str]
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/TradingPartnerComponent/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return TradingPartnerComponent._unmap(response)
        if content == "application/xml":
            return TradingPartnerComponent._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def update_trading_partner_component(
        self, id_: str, request_body: TradingPartnerComponent = None
    ) -> Union[TradingPartnerComponent, str]:
        """This operation overwrites the Trading Partner Component object with the specified component ID except as described:
         - If the fields are empty, an UPDATE operation specifying the organizationId field populates the ContactInformation fields with the data from the referenced Organization component. However, if those fields have values, they are not overwritten.
         An UPDATE operation specifying the ID of a deleted Trading Partner component restores the component to a non-deleted state, assuming the request is otherwise valid.

        :param request_body: The request body., defaults to None
        :type request_body: TradingPartnerComponent, optional
        :param id_: id_
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[TradingPartnerComponent, str]
        """

        Validator(TradingPartnerComponent).is_optional().validate(request_body)
        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/TradingPartnerComponent/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return TradingPartnerComponent._unmap(response)
        if content == "application/xml":
            return TradingPartnerComponent._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def delete_trading_partner_component(self, id_: str) -> None:
        """The DELETE operation deletes the Trading Partner Component object with a specific component ID.
         A DELETE operation specifying the ID of a deleted Trading Partner component returns a false response.

        :param id_: id_
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/TradingPartnerComponent/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("DELETE")
        )

        response, status, _ = self.send_request(serialized_request)

    @cast_models
    def bulk_trading_partner_component(
        self, request_body: TradingPartnerComponentBulkRequest = None
    ) -> Union[TradingPartnerComponentBulkResponse, str]:
        """The bulk GET operation returns multiple Trading Partner Component objects based on the supplied IDs, to a maximum of 100.

        :param request_body: The request body., defaults to None
        :type request_body: TradingPartnerComponentBulkRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[TradingPartnerComponentBulkResponse, str]
        """

        Validator(TradingPartnerComponentBulkRequest).is_optional().validate(
            request_body
        )

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/TradingPartnerComponent/bulk",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return TradingPartnerComponentBulkResponse._unmap(response)
        if content == "application/xml":
            return TradingPartnerComponentBulkResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_trading_partner_component(
        self, request_body: TradingPartnerComponentQueryConfig = None
    ) -> Union[TradingPartnerComponentQueryResponse, str]:
        """The QUERY operation returns each Trading Partner component that meets the specified filtering criteria.

         - The name field in a QUERY filter represents the object’s componentName field.
         - Only the LIKE operator is allowed with a name filter. Likewise, you can only use the EQUALS operator with a classification, standard, identifier filter, or a communication method filter (as2, disk, ftp, http, mllp, sftp). Filtering on a communication method field requests Trading Partner components by defining the communication method.
         - If the QUERY request includes multiple filters, you can connect the filters with a logical AND operator. The QUERY request does not support the logical OR operator.
         - The QUERY results omit the folderName field.

         For general information about the structure of QUERY filters, their sample payloads, and how to handle the paged results, refer to [Query filters](#section/Introduction/Query-filters) and [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body., defaults to None
        :type request_body: TradingPartnerComponentQueryConfig, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[TradingPartnerComponentQueryResponse, str]
        """

        Validator(TradingPartnerComponentQueryConfig).is_optional().validate(
            request_body
        )

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/TradingPartnerComponent/query",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return TradingPartnerComponentQueryResponse._unmap(response)
        if content == "application/xml":
            return TradingPartnerComponentQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_more_trading_partner_component(
        self, request_body: str
    ) -> Union[TradingPartnerComponentQueryResponse, str]:
        """To learn about using `queryMore`, refer to [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body.
        :type request_body: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[TradingPartnerComponentQueryResponse, str]
        """

        Validator(str).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/TradingPartnerComponent/queryMore",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body, "text/plain")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return TradingPartnerComponentQueryResponse._unmap(response)
        if content == "application/xml":
            return TradingPartnerComponentQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from __future__ import annotations
from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .ftpssl_options import FtpsslOptions


class ConnectionMode(Enum):
    """An enumeration representing different categories.

    :cvar ACTIVE: "active"
    :vartype ACTIVE: str
    :cvar PASSIVE: "passive"
    :vartype PASSIVE: str
    """

    ACTIVE = "active"
    PASSIVE = "passive"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, ConnectionMode._member_map_.values()))


@JsonMap(
    {
        "ftpssl_options": "FTPSSLOptions",
        "connection_mode": "connectionMode",
        "use_default_settings": "useDefaultSettings",
    }
)
class FtpSettings(BaseModel):
    """FtpSettings

    :param ftpssl_options: ftpssl_options, defaults to None
    :type ftpssl_options: FtpsslOptions, optional
    :param connection_mode: connection_mode, defaults to None
    :type connection_mode: ConnectionMode, optional
    :param host: host, defaults to None
    :type host: str, optional
    :param password: password, defaults to None
    :type password: str, optional
    :param port: port, defaults to None
    :type port: int, optional
    :param use_default_settings: use_default_settings, defaults to None
    :type use_default_settings: bool, optional
    :param user: user, defaults to None
    :type user: str, optional
    """

    def __init__(
        self,
        connection_mode: ConnectionMode = SENTINEL,
        ftpssl_options: FtpsslOptions = SENTINEL,
        host: str = SENTINEL,
        password: str = SENTINEL,
        port: int = SENTINEL,
        use_default_settings: bool = SENTINEL,
        user: str = SENTINEL,
        **kwargs,
    ):
        """FtpSettings

        :param ftpssl_options: ftpssl_options, defaults to None
        :type ftpssl_options: FtpsslOptions, optional
        :param connection_mode: connection_mode, defaults to None
        :type connection_mode: ConnectionMode, optional
        :param host: host, defaults to None
        :type host: str, optional
        :param password: password, defaults to None
        :type password: str, optional
        :param port: port, defaults to None
        :type port: int, optional
        :param use_default_settings: use_default_settings, defaults to None
        :type use_default_settings: bool, optional
        :param user: user, defaults to None
        :type user: str, optional
        """
        if ftpssl_options is not SENTINEL:
            self.ftpssl_options = self._define_object(ftpssl_options, FtpsslOptions)
        if connection_mode is not SENTINEL:
            self.connection_mode = self._enum_matching(
                connection_mode, ConnectionMode.list(), "connection_mode"
            )
        if host is not SENTINEL:
            self.host = host
        if password is not SENTINEL:
            self.password = password
        if port is not SENTINEL:
            self.port = port
        if use_default_settings is not SENTINEL:
            self.use_default_settings = use_default_settings
        if user is not SENTINEL:
            self.user = user
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..runtime_restart_request import RuntimeRestartRequestService
from ...models import RuntimeRestartRequest


class RuntimeRestartRequestServiceAsync(RuntimeRestartRequestService):
    """
    Async Wrapper for RuntimeRestartRequestServiceAsync
    """

    def create_runtime_restart_request(
        self, request_body: RuntimeRestartRequest = None
    ) -> Awaitable[Union[RuntimeRestartRequest, str]]:
        return to_async(super().create_runtime_restart_request)(request_body)




from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class ComponentMetadataType(Enum):
    """An enumeration representing different categories.

    :cvar CERTIFICATE: "certificate"
    :vartype CERTIFICATE: str
    :cvar CONNECTORACTION: "connector-action"
    :vartype CONNECTORACTION: str
    :cvar CONNECTORSETTINGS: "connector-settings"
    :vartype CONNECTORSETTINGS: str
    :cvar CROSSREF: "crossref"
    :vartype CROSSREF: str
    :cvar DOCUMENTCACHE: "documentcache"
    :vartype DOCUMENTCACHE: str
    :cvar TRANSFORMMAP: "transform.map"
    :vartype TRANSFORMMAP: str
    :cvar TRANSFORMFUNCTION: "transform.function"
    :vartype TRANSFORMFUNCTION: str
    :cvar CERTIFICATEPGP: "certificate.pgp"
    :vartype CERTIFICATEPGP: str
    :cvar PROCESS: "process"
    :vartype PROCESS: str
    :cvar PROCESSPROPERTY: "processproperty"
    :vartype PROCESSPROPERTY: str
    :cvar PROFILEDB: "profile.db"
    :vartype PROFILEDB: str
    :cvar PROFILEEDI: "profile.edi"
    :vartype PROFILEEDI: str
    :cvar PROFILEFLATFILE: "profile.flatfile"
    :vartype PROFILEFLATFILE: str
    :cvar PROFILEXML: "profile.xml"
    :vartype PROFILEXML: str
    :cvar PROFILEJSON: "profile.json"
    :vartype PROFILEJSON: str
    :cvar QUEUE: "queue"
    :vartype QUEUE: str
    :cvar TRADINGPARTNER: "tradingpartner"
    :vartype TRADINGPARTNER: str
    :cvar TPGROUP: "tpgroup"
    :vartype TPGROUP: str
    :cvar TPORGANIZATION: "tporganization"
    :vartype TPORGANIZATION: str
    :cvar TPCOMMOPTIONS: "tpcommoptions"
    :vartype TPCOMMOPTIONS: str
    :cvar WEBSERVICE: "webservice"
    :vartype WEBSERVICE: str
    :cvar WEBSERVICEEXTERNAL: "webservice.external"
    :vartype WEBSERVICEEXTERNAL: str
    :cvar PROCESSROUTE: "processroute"
    :vartype PROCESSROUTE: str
    :cvar CUSTOMLIBRARY: "customlibrary"
    :vartype CUSTOMLIBRARY: str
    :cvar EDISTANDARD: "edistandard"
    :vartype EDISTANDARD: str
    :cvar FLOWSERVICE: "flowservice"
    :vartype FLOWSERVICE: str
    :cvar SCRIPTPROCESSING: "script.processing"
    :vartype SCRIPTPROCESSING: str
    :cvar SCRIPTMAPPING: "script.mapping"
    :vartype SCRIPTMAPPING: str
    :cvar XSLT: "xslt"
    :vartype XSLT: str
    """

    CERTIFICATE = "certificate"
    CONNECTORACTION = "connector-action"
    CONNECTORSETTINGS = "connector-settings"
    CROSSREF = "crossref"
    DOCUMENTCACHE = "documentcache"
    TRANSFORMMAP = "transform.map"
    TRANSFORMFUNCTION = "transform.function"
    CERTIFICATEPGP = "certificate.pgp"
    PROCESS = "process"
    PROCESSPROPERTY = "processproperty"
    PROFILEDB = "profile.db"
    PROFILEEDI = "profile.edi"
    PROFILEFLATFILE = "profile.flatfile"
    PROFILEXML = "profile.xml"
    PROFILEJSON = "profile.json"
    QUEUE = "queue"
    TRADINGPARTNER = "tradingpartner"
    TPGROUP = "tpgroup"
    TPORGANIZATION = "tporganization"
    TPCOMMOPTIONS = "tpcommoptions"
    WEBSERVICE = "webservice"
    WEBSERVICEEXTERNAL = "webservice.external"
    PROCESSROUTE = "processroute"
    CUSTOMLIBRARY = "customlibrary"
    EDISTANDARD = "edistandard"
    FLOWSERVICE = "flowservice"
    SCRIPTPROCESSING = "script.processing"
    SCRIPTMAPPING = "script.mapping"
    XSLT = "xslt"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, ComponentMetadataType._member_map_.values()))


@JsonMap(
    {
        "branch_id": "branchId",
        "branch_name": "branchName",
        "component_id": "componentId",
        "copied_from_component_id": "copiedFromComponentId",
        "copied_from_component_version": "copiedFromComponentVersion",
        "created_by": "createdBy",
        "created_date": "createdDate",
        "current_version": "currentVersion",
        "folder_id": "folderId",
        "folder_name": "folderName",
        "modified_by": "modifiedBy",
        "modified_date": "modifiedDate",
        "sub_type": "subType",
        "type_": "type",
    }
)
class ComponentMetadata(BaseModel):
    """ComponentMetadata

    :param branch_id: If specified, the branch on which you want to manage the component., defaults to None
    :type branch_id: str, optional
    :param branch_name: branch_name, defaults to None
    :type branch_name: str, optional
    :param component_id: Required. Read only. The ID of the component. The component ID is available on the Revision History dialog, which you can access from the Build page in the service., defaults to None
    :type component_id: str, optional
    :param copied_from_component_id: Read only. If you copied the component, this field is the ID of the original component from where you copied the component., defaults to None
    :type copied_from_component_id: str, optional
    :param copied_from_component_version: Read only. If you copied the component, this field is the revision number of the original component you copied., defaults to None
    :type copied_from_component_version: int, optional
    :param created_by: Read only. User name of the user who created the component., defaults to None
    :type created_by: str, optional
    :param created_date: Read only. Date and time., defaults to None
    :type created_date: str, optional
    :param current_version: Read only. Indicates if the value specified in the version field is the most current and latest revision number created for the component on the **Build** tab. A value of True indicates that the revision number is the most current revision number on the **Build** tab, whereas False indicates that the version field value is not the most current revision number., defaults to None
    :type current_version: bool, optional
    :param deleted: Read only. Indicates if the component is deleted. A value of True indicates a deleted status, whereas False indicates an active status., defaults to None
    :type deleted: bool, optional
    :param folder_id: The ID of the folder where the component currently resides., defaults to None
    :type folder_id: str, optional
    :param folder_name: Read only. The folder location of the component within Component Explorer., defaults to None
    :type folder_name: str, optional
    :param modified_by: Read only. User name of the user who last modified the component., defaults to None
    :type modified_by: str, optional
    :param modified_date: Read only. Date and time., defaults to None
    :type modified_date: str, optional
    :param name: Read only., defaults to None
    :type name: str, optional
    :param sub_type: Read only. Used by connector-related components \(connections and operations\) to identify the connector type. Subtype values are the internal connector ID, not the user-facing name.See [Connector object](/api/platformapi#tag/Connector)., defaults to None
    :type sub_type: str, optional
    :param type_: Read only. The type of component. See the section **Component Types** later in this topic for a complete list of component type values, defaults to None
    :type type_: ComponentMetadataType, optional
    :param version: Read only., defaults to None
    :type version: int, optional
    """

    def __init__(
        self,
        branch_id: str = SENTINEL,
        branch_name: str = SENTINEL,
        component_id: str = SENTINEL,
        copied_from_component_id: str = SENTINEL,
        copied_from_component_version: int = SENTINEL,
        created_by: str = SENTINEL,
        created_date: str = SENTINEL,
        current_version: bool = SENTINEL,
        deleted: bool = SENTINEL,
        folder_id: str = SENTINEL,
        folder_name: str = SENTINEL,
        modified_by: str = SENTINEL,
        modified_date: str = SENTINEL,
        name: str = SENTINEL,
        sub_type: str = SENTINEL,
        type_: ComponentMetadataType = SENTINEL,
        version: int = SENTINEL,
        **kwargs
    ):
        """ComponentMetadata

        :param branch_id: If specified, the branch on which you want to manage the component., defaults to None
        :type branch_id: str, optional
        :param branch_name: branch_name, defaults to None
        :type branch_name: str, optional
        :param component_id: Required. Read only. The ID of the component. The component ID is available on the Revision History dialog, which you can access from the Build page in the service., defaults to None
        :type component_id: str, optional
        :param copied_from_component_id: Read only. If you copied the component, this field is the ID of the original component from where you copied the component., defaults to None
        :type copied_from_component_id: str, optional
        :param copied_from_component_version: Read only. If you copied the component, this field is the revision number of the original component you copied., defaults to None
        :type copied_from_component_version: int, optional
        :param created_by: Read only. User name of the user who created the component., defaults to None
        :type created_by: str, optional
        :param created_date: Read only. Date and time., defaults to None
        :type created_date: str, optional
        :param current_version: Read only. Indicates if the value specified in the version field is the most current and latest revision number created for the component on the **Build** tab. A value of True indicates that the revision number is the most current revision number on the **Build** tab, whereas False indicates that the version field value is not the most current revision number., defaults to None
        :type current_version: bool, optional
        :param deleted: Read only. Indicates if the component is deleted. A value of True indicates a deleted status, whereas False indicates an active status., defaults to None
        :type deleted: bool, optional
        :param folder_id: The ID of the folder where the component currently resides., defaults to None
        :type folder_id: str, optional
        :param folder_name: Read only. The folder location of the component within Component Explorer., defaults to None
        :type folder_name: str, optional
        :param modified_by: Read only. User name of the user who last modified the component., defaults to None
        :type modified_by: str, optional
        :param modified_date: Read only. Date and time., defaults to None
        :type modified_date: str, optional
        :param name: Read only., defaults to None
        :type name: str, optional
        :param sub_type: Read only. Used by connector-related components \(connections and operations\) to identify the connector type. Subtype values are the internal connector ID, not the user-facing name.See [Connector object](/api/platformapi#tag/Connector)., defaults to None
        :type sub_type: str, optional
        :param type_: Read only. The type of component. See the section **Component Types** later in this topic for a complete list of component type values, defaults to None
        :type type_: ComponentMetadataType, optional
        :param version: Read only., defaults to None
        :type version: int, optional
        """
        if branch_id is not SENTINEL:
            self.branch_id = branch_id
        if branch_name is not SENTINEL:
            self.branch_name = branch_name
        if component_id is not SENTINEL:
            self.component_id = component_id
        if copied_from_component_id is not SENTINEL:
            self.copied_from_component_id = copied_from_component_id
        if copied_from_component_version is not SENTINEL:
            self.copied_from_component_version = copied_from_component_version
        if created_by is not SENTINEL:
            self.created_by = created_by
        if created_date is not SENTINEL:
            self.created_date = created_date
        if current_version is not SENTINEL:
            self.current_version = current_version
        if deleted is not SENTINEL:
            self.deleted = deleted
        if folder_id is not SENTINEL:
            self.folder_id = folder_id
        if folder_name is not SENTINEL:
            self.folder_name = folder_name
        if modified_by is not SENTINEL:
            self.modified_by = modified_by
        if modified_date is not SENTINEL:
            self.modified_date = modified_date
        if name is not SENTINEL:
            self.name = name
        if sub_type is not SENTINEL:
            self.sub_type = sub_type
        if type_ is not SENTINEL:
            self.type_ = self._enum_matching(
                type_, ComponentMetadataType.list(), "type_"
            )
        if version is not SENTINEL:
            self.version = version
        self._kwargs = kwargs




from __future__ import annotations
from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .connector_fields import ConnectorFields
from .tracked_fields import TrackedFields


class GenericConnectorRecordStatus(Enum):
    """An enumeration representing different categories.

    :cvar SUCCESS: "SUCCESS"
    :vartype SUCCESS: str
    :cvar ERROR: "ERROR"
    :vartype ERROR: str
    """

    SUCCESS = "SUCCESS"
    ERROR = "ERROR"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, GenericConnectorRecordStatus._member_map_.values())
        )


@JsonMap(
    {
        "action_type": "actionType",
        "atom_id": "atomId",
        "connection_id": "connectionId",
        "connection_name": "connectionName",
        "connector_fields": "connectorFields",
        "connector_type": "connectorType",
        "date_processed": "dateProcessed",
        "document_index": "documentIndex",
        "error_message": "errorMessage",
        "execution_connector_id": "executionConnectorId",
        "execution_id": "executionId",
        "id_": "id",
        "incremental_document_index": "incrementalDocumentIndex",
        "operation_id": "operationId",
        "operation_name": "operationName",
        "start_shape": "startShape",
        "tracked_fields": "trackedFields",
    }
)
class GenericConnectorRecord(BaseModel):
    """GenericConnectorRecord

    :param account: account, defaults to None
    :type account: str, optional
    :param action_type: The type of action the connector performs, for example, GET, SEND, LISTEN, and so on., defaults to None
    :type action_type: str, optional
    :param atom_id: The ID of the Runtime in which the process ran. In the case of a Runtime cloud, use the ID of the Cloud attachment and not that of the Cloud itself., defaults to None
    :type atom_id: str, optional
    :param connection_id: When sending documents into or out of the Connection component, use this ID., defaults to None
    :type connection_id: str, optional
    :param connection_name: When sending documents into or out of the Connection component, use this user-defined name., defaults to None
    :type connection_name: str, optional
    :param connector_fields: Displays all connector-related fields from the connector included in this document.
    :type connector_fields: ConnectorFields
    :param connector_type: The internal and unique identifier for the connector type the document was sent into or out of, such as `http`, `ftp`, or `greatplains`., defaults to None
    :type connector_type: str, optional
    :param date_processed: The processing date and time of the document., defaults to None
    :type date_processed: str, optional
    :param document_index: Index of the document within the context of its execution connector (starting from 0)., defaults to None
    :type document_index: int, optional
    :param error_message: Displays the corresponding error message for an unsuccessful document.
    :type error_message: str
    :param execution_connector_id: The ID of the [Execution Connector object](/api/platformapi#tag/ExecutionConnector). This ID identifies the execution connector of which this document is a part., defaults to None
    :type execution_connector_id: str, optional
    :param execution_id: The ID of the process run., defaults to None
    :type execution_id: str, optional
    :param id_: The ID of the GenericConnectorRecord. You obtain this ID from querying the GenericConnectorRecord object, defaults to None
    :type id_: str, optional
    :param incremental_document_index: Index of the document in the context of the overall run (starting from 1)., defaults to None
    :type incremental_document_index: int, optional
    :param operation_id: When sending documents into or out of the Operation component, use this ID., defaults to None
    :type operation_id: str, optional
    :param operation_name: When sending documents into or out of the Operation component, use this user-defined name., defaults to None
    :type operation_name: str, optional
    :param retryable: If the value is true, this indicates that you can rerun the document using the Rerun Document operation. If the value is false, this indicates that you cannot rerun the document using the Rerun Document operation., defaults to None
    :type retryable: bool, optional
    :param size: The size of the document in kilobytes., defaults to None
    :type size: int, optional
    :param start_shape: If the value is true, this indicates the configuration of the Connector or Trading Partner as a **Start** shape in the process run. If the value is false, this indicates that you did not configure the Connector or Trading Partner as a **Start** shape in the process run., defaults to None
    :type start_shape: bool, optional
    :param status: Indicates whether the document successfully or unsuccessfully ran., defaults to None
    :type status: GenericConnectorRecordStatus, optional
    :param tracked_fields: Displays all the custom tracked fields from this document.
    :type tracked_fields: TrackedFields
    """

    def __init__(
        self,
        account: str = SENTINEL,
        action_type: str = SENTINEL,
        atom_id: str = SENTINEL,
        connection_id: str = SENTINEL,
        connection_name: str = SENTINEL,
        connector_fields: ConnectorFields = SENTINEL,
        connector_type: str = SENTINEL,
        date_processed: str = SENTINEL,
        document_index: int = SENTINEL,
        error_message: str = SENTINEL,
        execution_connector_id: str = SENTINEL,
        execution_id: str = SENTINEL,
        id_: str = SENTINEL,
        incremental_document_index: int = SENTINEL,
        operation_id: str = SENTINEL,
        operation_name: str = SENTINEL,
        retryable: bool = SENTINEL,
        size: int = SENTINEL,
        start_shape: bool = SENTINEL,
        status: GenericConnectorRecordStatus = SENTINEL,
        tracked_fields: TrackedFields = SENTINEL,
        **kwargs,
    ):
        """GenericConnectorRecord

        :param account: account, defaults to None
        :type account: str, optional
        :param action_type: The type of action the connector performs, for example, GET, SEND, LISTEN, and so on., defaults to None
        :type action_type: str, optional
        :param atom_id: The ID of the Runtime in which the process ran. In the case of a Runtime cloud, use the ID of the Cloud attachment and not that of the Cloud itself., defaults to None
        :type atom_id: str, optional
        :param connection_id: When sending documents into or out of the Connection component, use this ID., defaults to None
        :type connection_id: str, optional
        :param connection_name: When sending documents into or out of the Connection component, use this user-defined name., defaults to None
        :type connection_name: str, optional
        :param connector_fields: Displays all connector-related fields from the connector included in this document.
        :type connector_fields: ConnectorFields
        :param connector_type: The internal and unique identifier for the connector type the document was sent into or out of, such as `http`, `ftp`, or `greatplains`., defaults to None
        :type connector_type: str, optional
        :param date_processed: The processing date and time of the document., defaults to None
        :type date_processed: str, optional
        :param document_index: Index of the document within the context of its execution connector (starting from 0)., defaults to None
        :type document_index: int, optional
        :param error_message: Displays the corresponding error message for an unsuccessful document.
        :type error_message: str
        :param execution_connector_id: The ID of the [Execution Connector object](/api/platformapi#tag/ExecutionConnector). This ID identifies the execution connector of which this document is a part., defaults to None
        :type execution_connector_id: str, optional
        :param execution_id: The ID of the process run., defaults to None
        :type execution_id: str, optional
        :param id_: The ID of the GenericConnectorRecord. You obtain this ID from querying the GenericConnectorRecord object, defaults to None
        :type id_: str, optional
        :param incremental_document_index: Index of the document in the context of the overall run (starting from 1)., defaults to None
        :type incremental_document_index: int, optional
        :param operation_id: When sending documents into or out of the Operation component, use this ID., defaults to None
        :type operation_id: str, optional
        :param operation_name: When sending documents into or out of the Operation component, use this user-defined name., defaults to None
        :type operation_name: str, optional
        :param retryable: If the value is true, this indicates that you can rerun the document using the Rerun Document operation. If the value is false, this indicates that you cannot rerun the document using the Rerun Document operation., defaults to None
        :type retryable: bool, optional
        :param size: The size of the document in kilobytes., defaults to None
        :type size: int, optional
        :param start_shape: If the value is true, this indicates the configuration of the Connector or Trading Partner as a **Start** shape in the process run. If the value is false, this indicates that you did not configure the Connector or Trading Partner as a **Start** shape in the process run., defaults to None
        :type start_shape: bool, optional
        :param status: Indicates whether the document successfully or unsuccessfully ran., defaults to None
        :type status: GenericConnectorRecordStatus, optional
        :param tracked_fields: Displays all the custom tracked fields from this document.
        :type tracked_fields: TrackedFields
        """
        if account is not SENTINEL:
            self.account = account
        if action_type is not SENTINEL:
            self.action_type = action_type
        if atom_id is not SENTINEL:
            self.atom_id = atom_id
        if connection_id is not SENTINEL:
            self.connection_id = connection_id
        if connection_name is not SENTINEL:
            self.connection_name = connection_name
        if connector_fields is not SENTINEL:
            self.connector_fields = self._define_object(connector_fields, ConnectorFields)
        if connector_type is not SENTINEL:
            self.connector_type = connector_type
        if date_processed is not SENTINEL:
            self.date_processed = date_processed
        if document_index is not SENTINEL:
            self.document_index = document_index
        if error_message is not SENTINEL:
            self.error_message = error_message
        if execution_connector_id is not SENTINEL:
            self.execution_connector_id = execution_connector_id
        if execution_id is not SENTINEL:
            self.execution_id = execution_id
        if id_ is not SENTINEL:
            self.id_ = id_
        if incremental_document_index is not SENTINEL:
            self.incremental_document_index = incremental_document_index
        if operation_id is not SENTINEL:
            self.operation_id = operation_id
        if operation_name is not SENTINEL:
            self.operation_name = operation_name
        if retryable is not SENTINEL:
            self.retryable = retryable
        if size is not SENTINEL:
            self.size = size
        if start_shape is not SENTINEL:
            self.start_shape = start_shape
        if status is not SENTINEL:
            self.status = self._enum_matching(
                status, GenericConnectorRecordStatus.list(), "status"
            )
        if tracked_fields is not SENTINEL:
            self.tracked_fields = self._define_object(tracked_fields, TrackedFields)
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({"execution_id": "executionId"})
class ExecutionArtifacts(BaseModel):
    """ExecutionArtifacts

    :param execution_id: The ID of the given process run. You can access the run ID in the View Extended Information dialog (Manage \> Process Reporting \> Action menu) on the user interface., defaults to None
    :type execution_id: str, optional
    """

    def __init__(self, execution_id: str = SENTINEL, **kwargs):
        """ExecutionArtifacts

        :param execution_id: The ID of the given process run. You can access the run ID in the View Extended Information dialog (Manage \> Process Reporting \> Action menu) on the user interface., defaults to None
        :type execution_id: str, optional
        """
        if execution_id is not SENTINEL:
            self.execution_id = execution_id
        self._kwargs = kwargs




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .processing_group_partner_standard_route import ProcessingGroupPartnerStandardRoute


@JsonMap(
    {
        "standard_route": "StandardRoute",
        "process_id": "processId",
        "trading_partner_id": "tradingPartnerId",
    }
)
class ProcessingGroupPartnerRoute(BaseModel):
    """ProcessingGroupPartnerRoute

    :param standard_route: standard_route, defaults to None
    :type standard_route: List[ProcessingGroupPartnerStandardRoute], optional
    :param process_id: process_id, defaults to None
    :type process_id: str, optional
    :param trading_partner_id: trading_partner_id, defaults to None
    :type trading_partner_id: str, optional
    """

    def __init__(
        self,
        standard_route: List[ProcessingGroupPartnerStandardRoute] = SENTINEL,
        process_id: str = SENTINEL,
        trading_partner_id: str = SENTINEL,
        **kwargs,
    ):
        """ProcessingGroupPartnerRoute

        :param standard_route: standard_route, defaults to None
        :type standard_route: List[ProcessingGroupPartnerStandardRoute], optional
        :param process_id: process_id, defaults to None
        :type process_id: str, optional
        :param trading_partner_id: trading_partner_id, defaults to None
        :type trading_partner_id: str, optional
        """
        if standard_route is not SENTINEL:
            self.standard_route = self._define_list(
                standard_route, ProcessingGroupPartnerStandardRoute
            )
        if process_id is not SENTINEL:
            self.process_id = process_id
        if trading_partner_id is not SENTINEL:
            self.trading_partner_id = trading_partner_id
        self._kwargs = kwargs




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .runtime_cloud_simple_expression import RuntimeCloudSimpleExpression
from .runtime_cloud_grouping_expression import RuntimeCloudGroupingExpression


class RuntimeCloudExpressionGuard(OneOfBaseModel):
    class_list = {
        "RuntimeCloudSimpleExpression": RuntimeCloudSimpleExpression,
        "RuntimeCloudGroupingExpression": RuntimeCloudGroupingExpression,
    }


RuntimeCloudExpression = Union[RuntimeCloudSimpleExpression, RuntimeCloudGroupingExpression]




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .environment_extensions_simple_expression import (
    EnvironmentExtensionsSimpleExpression,
)
from .environment_extensions_grouping_expression import (
    EnvironmentExtensionsGroupingExpression,
)


class EnvironmentExtensionsExpressionGuard(OneOfBaseModel):
    class_list = {
        "EnvironmentExtensionsSimpleExpression": EnvironmentExtensionsSimpleExpression,
        "EnvironmentExtensionsGroupingExpression": EnvironmentExtensionsGroupingExpression,
    }


EnvironmentExtensionsExpression = Union[
    EnvironmentExtensionsSimpleExpression, EnvironmentExtensionsGroupingExpression
]
